/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2004 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/



//
// internal.c
//
// This is sample coding showing a simple use of Syntrillium Software's Cool
// Edit Pro 2.0 Controller SDK.
//
// This file implements the details for the sample and is called from the 
// raw API in sample.c
//
// Where possible the code is broken into two parts.  A generic "hardware"
// interface and the sample software implementation.  The implementation
// functions are prefixed with Impl and are at the end of the file.
//
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include "..\ctrlmsg.h"
#include "internal.h"
#include "resource.h"

// Module scope variables
static HWND hwndMain_;
static int selected_track_ = -1;
static ControllerFunctions cfn_;
static DWORD dwUserCommand_;
static BOOL active_ = FALSE;
const DWORD inValidCommand = -1;


// Module scope function prototypes
static LRESULT CALLBACK Impl_ControllerDlgProc(HWND, UINT, WPARAM, LPARAM);
static BOOL Impl_IsHardwareActive(void);
static void Impl_SetTrackVolume(double fVolDecible);
static void Impl_SetTrackName(LPCSTR szTitle);
static void LoadCustomSettings(void);


//////////////////////////////////////////////////////////////////////////////
/*! Initialize all needed resources and get the device ready for the user.
*/
BOOL Hardware_Open(
	HINSTANCE hInstance,
	ControllerFunctions *pcfn)
{
	// Save the function pointers for use later.  We can call these any time
	// to retrive information from CEP.
	cfn_ = *pcfn;

	LoadCustomSettings();

	// Implementation specific intialization
	if(!(hwndMain_ = CreateDialog(hInstance,
		MAKEINTRESOURCE(IDD_DIALOG1), 0, Impl_ControllerDlgProc)))
		return FALSE;
	UpdateWindow(hwndMain_);
	ShowWindow(hwndMain_, SW_NORMAL);

	active_ = TRUE;

	// Let CEP know that we are ready to go
	return TRUE;
} // Hardware_Open


//////////////////////////////////////////////////////////////////////////////
/*! Respond to any message from CEP that we care about.  Try not to take too
	long.  If we don't care about the message we just ignore it.
*/
void Hardware_Update(DWORD dwMsg, void *pVal1, void *pVal2, void *pVal3)
{
	if(!Impl_IsHardwareActive())
		return;

	// Track message - Volume Change
	if(dwMsg == CTRL_TRACK_FADER)
	{
		int track = (int)pVal1;
		if(track == selected_track_)
		{
			double fVol =  *((double *)pVal2);
			Impl_SetTrackVolume(fVol);
		}
	}
	// Track message - Active Track Changed
	else if(dwMsg == CTRL_TRACK_SELECT)
	{
		double fVol;
		char szTitle[MAX_TRACK_TITLE];
		selected_track_ = (int)pVal1;
		// Get the track name
		GET_TRACK_TITLE(cfn_.pGetStatus, selected_track_, szTitle);
		Impl_SetTrackName(szTitle);

		// Get the track volume
		GET_TRACK_FADER(cfn_.pGetStatus, selected_track_, &fVol);
		Impl_SetTrackVolume(fVol);
	}
	else if (dwMsg == CTRL_REGISTRY_CHANGE)
	{
		LoadCustomSettings();
	}

} // Hardware_Update


//////////////////////////////////////////////////////////////////////////////
/*! Close any resources we are using.
*/
void Hardware_Close(void)
{
	if(hwndMain_)
		DestroyWindow(hwndMain_);

	active_ = FALSE;
} // Hardware_Close


//////////////////////////////////////////////////////////////////////////////
// From here on out are implementation specific functions.  There is little
// of interested beyond this point.
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
BOOL Impl_IsHardwareActive(void)
{
	return hwndMain_ != 0;
} // Impl_IsHardwareActive



//////////////////////////////////////////////////////////////////////////////
void Impl_SetTrackVolume(double fVolDecible)
{
	// Convert from db to integral 0-100 for our slider control.
	const double range = MAXTRACKVOLUME_DB - MINTRACKVOLUME_DB;
	const double fVolNorm = (fVolDecible - MINTRACKVOLUME_DB) / range;
	const HWND slider = GetDlgItem(hwndMain_, IDC_SLIDER1);
	if(!slider)
		return;
	SendMessage(slider, TBM_SETPOS, TRUE, (int)(fVolNorm * 100 + 0.5f));
} // Impl_SetTrackVolume


//////////////////////////////////////////////////////////////////////////////
void Impl_SetTrackName(LPCSTR szTitle)
{
	if(!hwndMain_)
		return;
	SetDlgItemText(hwndMain_, IDC_EDIT1, szTitle);
} // Impl_SetTrackName


//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK Impl_ControllerDlgProc
	(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_INITDIALOG:
			ShowWindow(GetDlgItem(hwnd, IDC_COMBO1), SW_HIDE);
			SendDlgItemMessage(hwnd,
				IDC_SLIDER1, TBM_SETRANGE, (WPARAM)FALSE, MAKELONG(0, 100));
			return TRUE;

		case WM_DESTROY:
			hwndMain_ = 0;
			break;

		case WM_COMMAND:
		{
			const WORD control = LOWORD(wParam);
			if(control == IDC_BUTTON1)
			{	// Play
				SET_DECK_PLAY(cfn_.pControlProc);
			}
			else if(control == IDC_BUTTON2)
			{	// Stop
				SET_DECK_STOP(cfn_.pControlProc);
			}
			else if(control == IDC_CHECK3)
			{	// User Button
				if(dwUserCommand_ != inValidCommand)
				{
					cfn_.ExecuteCommand(dwUserCommand_, (WORD)(IsDlgButtonChecked
						(hwnd, IDC_CHECK3) ? CONTROLLER_KEYDOWN : CONTROLLER_KEYUP));
				}
			}
			else if(control == IDC_CHECK1)
			{	// RR
				SET_DECK_REW(cfn_.pControlProc, IsDlgButtonChecked(hwnd, IDC_CHECK1));
			}
			else if(control == IDC_CHECK2)
			{	// FF
				SET_DECK_FFW(cfn_.pControlProc, IsDlgButtonChecked(hwnd, IDC_CHECK2));
			}
		}
		
		case WM_HSCROLL:
		{
			int code = LOWORD(wParam);
			int position = HIWORD(wParam);
			if(code == TB_THUMBTRACK)
			{
			/*
				The user has moved the slider and therefor changed the track
				volume.  We need to let CEP know about it.  This slider has an
				integral range from 1 to 100, so we need to convert to db.
			*/
				double fVolNorm = position / 100.0;
				double fVolDecible = fVolNorm *
					(MAXTRACKVOLUME_DB - MINTRACKVOLUME_DB)
					+ MINTRACKVOLUME_DB;
				SET_TRACK_FADER_F(cfn_.pControlProc, selected_track_, fVolDecible);
			}
			break;
		}

		case WM_NOTIFY:
		{
			const int idCtrl = (int)wParam;
			if(idCtrl == IDC_SLIDER1 && msg == TBN_ENDDRAG)
			{
				LPNMTBDISPINFO lptbdi = (LPNMTBDISPINFO)lParam;

				break;
			}
			else
				return FALSE;
		}

		default:
			return FALSE;
	}

	return TRUE;
} // Impl_ControllerDlgProc


//////////////////////////////////////////////////////////////////////////////
/*! DialogProc for the configuration dialog
	will be because the program is closing or the user selected a different
	controller.
*/
int CALLBACK ConfigDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_INITDIALOG:
		{
			HKEY hkey;
			int i;
			int iUID;
			char sz[128];
			HWND hwndList = GetDlgItem(hwnd, IDC_LIST1);
			DWORD dwUserCommand = inValidCommand;
			DWORD dwFlags;

			ConfigDialogProcParam* param = (ConfigDialogProcParam *)lParam;
			if(!param || !param->GetCommand)
				EndDialog(hwnd, 0);

			for(i = 0; i < param->count; ++i)
			{
				param->GetCommand(i, sz, 128, &iUID, &dwFlags);
				if (dwFlags)	// if zero, the command is not intended for public use
				{
					LRESULT newIndex = ListBox_AddString(hwndList, sz);
					ListBox_SetItemData(hwndList, newIndex, iUID);
				}
			}

			if(ERROR_SUCCESS == RegOpenKey(HKEY_CURRENT_USER,
				"Software\\YourCompany\\YourProgram", &hkey))
			{
				int userCommand = 0;
				DWORD cb = sizeof(userCommand);
				DWORD regType = REG_DWORD;
				if (ERROR_SUCCESS == RegQueryValueEx(hkey, "USER1", 0, &regType, (LPBYTE)&userCommand, &cb))
				{
					int found = ListBox_FindItemData(hwndList, -1, userCommand);
					if(found != LB_ERR)
					{
						ListBox_SetCurSel(hwndList, found);
					}
				}
								
				RegCloseKey(hkey);
			}

			return TRUE;
		}

		case WM_COMMAND:
		{
			const WORD control = LOWORD(wParam);
			if(control == IDOK)
			{
				HKEY hkey;
				DWORD dwDisposition;
				if(ERROR_SUCCESS == RegCreateKeyEx(HKEY_CURRENT_USER,
					"Software\\YourCompany\\YourProgram", 0, "",
					REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, &hkey,
					&dwDisposition))
				{
					HWND hwndList = GetDlgItem(hwnd, IDC_LIST1);
					int iUID = ListBox_GetItemData(hwndList, ListBox_GetCurSel(hwndList));
					RegSetValueEx(hkey, "USER1", 0, REG_DWORD, (BYTE*)&iUID, sizeof(iUID));
					RegCloseKey(hkey);
				}

				EndDialog(hwnd, 0);
			}
			else if(control == IDCANCEL)
				EndDialog(hwnd, 0);
			else
				return FALSE;
			break;
		}

		default:
			return FALSE;
			break;
	}

	return TRUE;
} // ConfigDlgProc


//////////////////////////////////////////////////////////////////////////////
void LoadCustomSettings()
{
	HKEY hkey;
	if(ERROR_SUCCESS == RegOpenKey(HKEY_CURRENT_USER,
		"Software\\YourCompany\\YourProgram", &hkey))
	{
		//char szUserCommand[128];
		//*szUserCommand = '\0';
		int userCommand = 0;
		DWORD cb = sizeof(userCommand);
		DWORD regType = REG_DWORD;
		if (ERROR_SUCCESS == RegQueryValueEx(hkey, "USER1", 0, &regType, (LPBYTE)&userCommand, &cb))
		{
			cfn_.LookupCommandIndex(userCommand, &dwUserCommand_);
		}

		RegCloseKey(hkey);
	}
} // LoadCustomSettings
