/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2006-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#define STRICT

#include "SessionSdkInterpretter.h"
#include <Windows.h>		// for CopyMemory and ZeroMemory


namespace AuditionSessionSDK
{


namespace
{

	template<typename T>
	void	EnsureBackwardCompatible(std::auto_ptr<T>& sdkInfo_myCopy, const T* & inSdkInfo)
	{
		// If input structure identifies it's size as less than we expect, then we copy whatever data
		// we can to a new temporary structure, and work with the temporary structure instead.
		if (inSdkInfo->mySize < sizeof(T))
		{
			sdkInfo_myCopy.reset(new T);
			::ZeroMemory(sdkInfo_myCopy.get(), sizeof(T));
			::CopyMemory(sdkInfo_myCopy.get(), inSdkInfo, inSdkInfo->mySize);
			inSdkInfo = sdkInfo_myCopy.get();
		}
	}

	std::wstring StringFromSdk(SDK_SESSION_STRING* inString)
	{
		std::wstring string;
		if (inString != NULL && inString->stringSize > 1)
		{
			string.assign(inString->chars, inString->stringSize - 1);
		}

		return string;
	}

	inline bool BoolFromSdk(const b32& inBool)
	{
		return inBool != 0;
	}

	GUID GuidFromSdk(const SDK_SESSION_GUID inGuid)
	{
		GUID standardGuid;
		standardGuid.Data1 = inGuid.Data1;
		standardGuid.Data2 = inGuid.Data2;
		standardGuid.Data3 = inGuid.Data3;
		for (int index = 0; index < 8; ++index)
		{
			standardGuid.Data4[index] = inGuid.Data4[index];
		}

		return standardGuid;
	}

	void EnvelopePointsFromSdk(SDK_SESSION_ENVELOPE_POINT_INFO** inPoints, u32 numPoints, std::vector<SESSION_ENVELOPE_POINT_INFO_NEW>& outPoints)
	{
		outPoints.clear();

		if (inPoints != NULL)
		{
			for (u32 index = 0; index != numPoints; ++index)
			{
				const SDK_SESSION_ENVELOPE_POINT_INFO* point = inPoints[index];
				if (point == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_ENVELOPE_POINT_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_ENVELOPE_POINT_INFO>(sdkInfo_myCopy, point);

				SESSION_ENVELOPE_POINT_INFO_NEW newPoint;
				newPoint.value = point->value;
				newPoint.dwAt = point->dwAt;

				outPoints.push_back(newPoint);
			}
		}
	}

	bool EnvelopeFromSdk(const SDK_SESSION_ENV_INFO* inEnvelope, SESSION_ENV_INFO_NEW& outEnvelope)
	{
		// First, check to see if we have any sdk information at all
		if (inEnvelope == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_ENV_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_ENV_INFO>(sdkInfo_myCopy, inEnvelope);

		// Convert between the SDK struct and our common struct
		outEnvelope.uiType = inEnvelope->uiType;
		outEnvelope.bSplined = BoolFromSdk(inEnvelope->bSplined);
		outEnvelope.szName = StringFromSdk(inEnvelope->szName);
		outEnvelope.dwFlags = inEnvelope->dwFlags;
		outEnvelope.LineColor = inEnvelope->LineColor;
		outEnvelope.dwSize = inEnvelope->dwSize;
		outEnvelope.bReadOnly = BoolFromSdk(inEnvelope->bReadOnly);
		EnvelopePointsFromSdk(inEnvelope->points, inEnvelope->numPoints, outEnvelope.points);

		return true;
	}

	void EnvelopesFromSdk(SDK_SESSION_ENV_INFO** inEnvelopes, u32 numEnvelopes, std::vector<SESSION_ENV_INFO_NEW>& outEnvelopes)
	{
		outEnvelopes.clear();

		if (inEnvelopes != NULL)
		{
			for (u32 index = 0; index != numEnvelopes; ++index)
			{
				SESSION_ENV_INFO_NEW newEnvelope;
				if (EnvelopeFromSdk(inEnvelopes[index], newEnvelope))
				{
					outEnvelopes.push_back(newEnvelope);
				}
			}
		}
	}

	bool AudioParamFromSdk(const SDK_SESSION_AUDIO_PARAM* inAudioParam, SESSION_AUDIO_PARAM_NEW& outAudioParam)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioParam == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_PARAM> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_PARAM>(sdkInfo_myCopy, inAudioParam);

		// Convert between the SDK struct and our common struct
		outAudioParam.bIsAutomatable = BoolFromSdk(inAudioParam->bIsAutomatable);
		outAudioParam.parameterValueForRendering = inAudioParam->parameterValueForRendering;
		outAudioParam.bSafeDuringWrite = BoolFromSdk(inAudioParam->bSafeDuringWrite);
		if (!EnvelopeFromSdk(inAudioParam->envelope, outAudioParam.envelope))
		{
			return false;
		}

		return true;
	}

	void AudioParamsFromSdk(SDK_SESSION_AUDIO_PARAM** inAudioParams, u32 numParams, std::vector<SESSION_AUDIO_PARAM_NEW>& outAudioParams)
	{
		outAudioParams.clear();

		if (inAudioParams != NULL)
		{
			for (u32 index = 0; index != numParams; ++index)
			{
				SESSION_AUDIO_PARAM_NEW newParam;
				if (AudioParamFromSdk(inAudioParams[index], newParam))
				{
					outAudioParams.push_back(newParam);
				}
			}
		}
	}

	bool AudioComponentFromSdk(const SDK_SESSION_AUDIO_COMPONENT* inAudioComponent, SESSION_AUDIO_COMPONENT_NEW& outAudioComponent)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioComponent == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_COMPONENT> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_COMPONENT>(sdkInfo_myCopy, inAudioComponent);

		// Convert between the SDK struct and our common struct
		outAudioComponent.componentType = inAudioComponent->componentType;
		outAudioComponent.initialDelay = inAudioComponent->initialDelay;
		outAudioComponent.isBypass = BoolFromSdk(inAudioComponent->isBypass);
		outAudioComponent.isSerial = BoolFromSdk(inAudioComponent->isSerial);
		AudioParamsFromSdk(inAudioComponent->audioParams, inAudioComponent->numParams, outAudioComponent.audioParams);

		return true;
	}

	bool AudioOutputComponentFromSdk(const SDK_SESSION_AUDIO_OUTPUT_COMPONENT* inAudioOutputComponent, SESSION_AUDIO_OUTPUT_COMPONENT_NEW& outAudioOutputComponent)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioOutputComponent == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_OUTPUT_COMPONENT> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_OUTPUT_COMPONENT>(sdkInfo_myCopy, inAudioOutputComponent);

		// Convert between the SDK struct and our common struct
		outAudioOutputComponent.componentType = inAudioOutputComponent->componentType;
		outAudioOutputComponent.initialDelay = inAudioOutputComponent->initialDelay;
		outAudioOutputComponent.isBypass = BoolFromSdk(inAudioOutputComponent->isBypass);
		AudioParamsFromSdk(inAudioOutputComponent->audioParams, inAudioOutputComponent->numParams, outAudioOutputComponent.audioParams);
		outAudioOutputComponent.pannerMode = inAudioOutputComponent->pannerMode;
		outAudioOutputComponent.isSumDown = BoolFromSdk(inAudioOutputComponent->isSumDown);
		outAudioOutputComponent.outputChannels = inAudioOutputComponent->outputChannels;
		outAudioOutputComponent.busID = inAudioOutputComponent->busID;

		return true;
	}

	bool AudioEffectComponentFromSdk(const SDK_SESSION_AUDIO_EFFECT_COMPONENT* inAudioComponent, SESSION_AUDIO_EFFECT_COMPONENT_NEW& outAudioComponent)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioComponent == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_EFFECT_COMPONENT> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_EFFECT_COMPONENT>(sdkInfo_myCopy, inAudioComponent);

		// Convert between the SDK struct and our common struct
		outAudioComponent.componentType = inAudioComponent->componentType;
		outAudioComponent.initialDelay = inAudioComponent->initialDelay;
		outAudioComponent.isBypass = BoolFromSdk(inAudioComponent->isBypass);
		AudioParamsFromSdk(inAudioComponent->audioParams, inAudioComponent->numParams, outAudioComponent.audioParams);
		outAudioComponent.guid = GuidFromSdk(inAudioComponent->guid);
		outAudioComponent.name = StringFromSdk(inAudioComponent->name);
		outAudioComponent.groupName = StringFromSdk(inAudioComponent->groupName);
		outAudioComponent.effectParamsString = StringFromSdk(inAudioComponent->effectParamsString);

		return true;
	}

	void AudioEffectComponentsFromSdk(SDK_SESSION_AUDIO_EFFECT_COMPONENT** inEffects, u32 numEffects, std::vector<SESSION_AUDIO_EFFECT_COMPONENT_NEW>& outEffects)
	{
		outEffects.clear();

		if (inEffects != NULL)
		{
			for (u32 index = 0; index != numEffects; ++index)
			{
				SESSION_AUDIO_EFFECT_COMPONENT_NEW newEffect;
				if (AudioEffectComponentFromSdk(inEffects[index], newEffect))
				{
					outEffects.push_back(newEffect);
				}
			}
		}
	}

	bool AudioSendComponentFromSdk(const SDK_SESSION_AUDIO_SEND_COMPONENT* inAudioComponent, SESSION_AUDIO_SEND_COMPONENT_NEW& outAudioComponent)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioComponent == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_SEND_COMPONENT> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_SEND_COMPONENT>(sdkInfo_myCopy, inAudioComponent);

		// Convert between the SDK struct and our common struct
		outAudioComponent.componentType = inAudioComponent->componentType;
		outAudioComponent.initialDelay = inAudioComponent->initialDelay;
		outAudioComponent.isBypass = BoolFromSdk(inAudioComponent->isBypass);
		AudioParamsFromSdk(inAudioComponent->audioParams, inAudioComponent->numParams, outAudioComponent.audioParams);
		outAudioComponent.pannerMode = inAudioComponent->pannerMode;
		outAudioComponent.isSumDown = BoolFromSdk(inAudioComponent->isSumDown);
		outAudioComponent.numOutputChannels = inAudioComponent->numOutputChannels;
		outAudioComponent.busID = inAudioComponent->busID;
		outAudioComponent.isPre = BoolFromSdk(inAudioComponent->isPre);

		return true;
	}

	void AudioSendComponentsFromSdk(SDK_SESSION_AUDIO_SEND_COMPONENT** inSends, u32 numSends, std::vector<SESSION_AUDIO_SEND_COMPONENT_NEW>& outSends)
	{
		outSends.clear();

		if (inSends != NULL)
		{
			for (u32 index = 0; index != numSends; ++index)
			{
				SESSION_AUDIO_SEND_COMPONENT_NEW newSend;
				if (AudioSendComponentFromSdk(inSends[index], newSend))
				{
					outSends.push_back(newSend);
				}
			}
		}
	}

	bool AudioEffectComponentGroupFromSdk(const SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP* inAudioComponent, SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW& outAudioComponent)
	{
		// First, check to see if we have any sdk information at all
		if (inAudioComponent == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP>(sdkInfo_myCopy, inAudioComponent);

		// Convert between the SDK struct and our common struct
		outAudioComponent.componentType = inAudioComponent->componentType;
		outAudioComponent.initialDelay = inAudioComponent->initialDelay;
		outAudioComponent.isBypass = BoolFromSdk(inAudioComponent->isBypass);
		outAudioComponent.isSerial = BoolFromSdk(inAudioComponent->isSerial);
		AudioParamsFromSdk(inAudioComponent->audioParams, inAudioComponent->numParams, outAudioComponent.audioParams);

		if (!AudioComponentFromSdk(inAudioComponent->inputGain, outAudioComponent.inputGain))
		{
			return false;
		}

		if (!AudioComponentFromSdk(inAudioComponent->outputGain, outAudioComponent.outputGain))
		{
			return false;
		}

		return true;
	}

	void TrackAutomationLanesFromSdk(SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO** inAutomationLanes, u32 numAutomationLanes, std::vector<SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW>& outAutomationLanes)
	{
		outAutomationLanes.clear();

		if (inAutomationLanes != NULL)
		{
			for (u32 index = 0; index != numAutomationLanes; ++index)
			{
				const SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO* lane = inAutomationLanes[index];
				if (lane == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO>(sdkInfo_myCopy, lane);

				SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW newLane;
				newLane.audioComponentType = lane->audioComponentType;
				newLane.guidString = StringFromSdk(lane->guidString);
				newLane.componentName = StringFromSdk(lane->componentName);
				newLane.parameterIndex = lane->parameterIndex;
				newLane.laneUIHeight = lane->laneUIHeight;

				outAutomationLanes.push_back(newLane);
			}
		}
	}

	bool CommonTrackInfoFromSdk(const SDK_SESSION_COMMON_TRACK_INFO* inTrackInfo, SESSION_COMMON_TRACK_INFO_NEW& outTrackInfo)
	{
		// First, check to see if we have any sdk information at all
		if (inTrackInfo == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_COMMON_TRACK_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_COMMON_TRACK_INFO>(sdkInfo_myCopy, inTrackInfo);

		// Convert between the SDK struct and our common struct
		outTrackInfo.id = inTrackInfo->id;
		outTrackInfo.dwFlags = inTrackInfo->dwFlags;
		outTrackInfo.szTitle = StringFromSdk(inTrackInfo->szTitle);
		outTrackInfo.dwTrackHeight = inTrackInfo->dwTrackHeight;

		return true;
	}

	bool AudioTrackInfoFromSdk(const SDK_SESSION_AUDIO_TRACK_INFO* inTrackInfo, SESSION_AUDIO_TRACK_INFO_NEW& outTrackInfo)
	{
		bool success = true;

		// First, check to see if we have any sdk information at all
		if (inTrackInfo == NULL)
		{
			success = false;
			return success;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_TRACK_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_TRACK_INFO>(sdkInfo_myCopy, inTrackInfo);

		// Convert between the SDK struct and our common struct
		success = success && CommonTrackInfoFromSdk(inTrackInfo->baseTrackInfo, outTrackInfo.baseTrackInfo);
		outTrackInfo.trackType = inTrackInfo->trackType;
		outTrackInfo.busID = inTrackInfo->busID;
		outTrackInfo.automationMode = inTrackInfo->automationMode;
		outTrackInfo.inputDriver = StringFromSdk(inTrackInfo->inputDriver);
		outTrackInfo.inputPort = inTrackInfo->inputPort;
		outTrackInfo.inputPortName = StringFromSdk(inTrackInfo->inputPortName);
		outTrackInfo.outputDriver = StringFromSdk(inTrackInfo->outputDriver);
		outTrackInfo.outputPort = inTrackInfo->outputPort;
		outTrackInfo.outputPortName = StringFromSdk(inTrackInfo->outputPortName);
		outTrackInfo.isPreMeter = BoolFromSdk(inTrackInfo->isPreMeter);
		outTrackInfo.isPreEffect = BoolFromSdk(inTrackInfo->isPreEffect);
		outTrackInfo.isFreeze = BoolFromSdk(inTrackInfo->isFreeze);
		outTrackInfo.isTrackEQUI = BoolFromSdk(inTrackInfo->isTrackEQUI);
		outTrackInfo.isEffectsRackUI = BoolFromSdk(inTrackInfo->isEffectsRackUI);
		success = success && AudioComponentFromSdk(inTrackInfo->inputComponent, outTrackInfo.inputComponent);
		success = success && AudioComponentFromSdk(inTrackInfo->eqComponent, outTrackInfo.eqComponent);
		success = success && AudioComponentFromSdk(inTrackInfo->faderComponent, outTrackInfo.faderComponent);
		success = success && AudioComponentFromSdk(inTrackInfo->muteComponent, outTrackInfo.muteComponent);
		success = success && AudioOutputComponentFromSdk(inTrackInfo->outputComponent, outTrackInfo.outputComponent);
		success = success && AudioEffectComponentGroupFromSdk(inTrackInfo->effectComponentGroup, outTrackInfo.effectComponentGroup);
		AudioSendComponentsFromSdk(inTrackInfo->sendComponents, inTrackInfo->numSends, outTrackInfo.sendComponents);
		AudioEffectComponentsFromSdk(inTrackInfo->effectComponents, inTrackInfo->numEffects, outTrackInfo.effectComponents);

		for (u32 index = 0; index < inTrackInfo->numEffectSlots; ++index)
		{
			outTrackInfo.hasEffectInSlot.push_back(BoolFromSdk(inTrackInfo->hasEffectInSlot[index]));
		}

		outTrackInfo.isShowingAutomationLanes = BoolFromSdk(inTrackInfo->isShowingAutomationLanes);
		outTrackInfo.trackHeight = inTrackInfo->trackHeight;

		TrackAutomationLanesFromSdk(inTrackInfo->automationLanes, inTrackInfo->numAutomationLanes, outTrackInfo.automationLanes);

		success = success && AudioEffectComponentFromSdk(inTrackInfo->midiHostEffect, outTrackInfo.midiHostEffect);

		return success;
	}

	bool FullCommonTrackInfoFromSdk(const SDK_SESSION_COMMON_TRACK_FULL_INFO* inTrackInfo, SESSION_COMMON_TRACK_FULL_INFO_NEW& outTrackInfo)
	{
		// First, check to see if we have any sdk information at all
		if (inTrackInfo == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_COMMON_TRACK_FULL_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_COMMON_TRACK_FULL_INFO>(sdkInfo_myCopy, inTrackInfo);

		// Convert between the SDK struct and our common struct
		outTrackInfo.trackType = inTrackInfo->trackType;
		outTrackInfo.trackOrderIndex = inTrackInfo->trackOrderIndex;
		return CommonTrackInfoFromSdk(inTrackInfo->track, outTrackInfo.track);
	}

	void FullCommonTracksInfoFromSdk(SDK_SESSION_COMMON_TRACK_FULL_INFO** inTracks, u32 numTracks, std::vector<SESSION_COMMON_TRACK_FULL_INFO_NEW>& outTracks)
	{
		outTracks.clear();

		if (inTracks != NULL)
		{
			for (u32 index = 0; index != numTracks; ++index)
			{
				SESSION_COMMON_TRACK_FULL_INFO_NEW newTrack;
				if (FullCommonTrackInfoFromSdk(inTracks[index], newTrack))
				{
					outTracks.push_back(newTrack);
				}
			}
		}
	}

	bool FullAudioTrackInfoFromSdk(const SDK_SESSION_AUDIO_TRACK_FULL_INFO* inTrackInfo, SESSION_AUDIO_TRACK_FULL_INFO_NEW& outTrackInfo)
	{
		// First, check to see if we have any sdk information at all
		if (inTrackInfo == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_AUDIO_TRACK_FULL_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_AUDIO_TRACK_FULL_INFO>(sdkInfo_myCopy, inTrackInfo);

		// Convert between the SDK struct and our common struct
		outTrackInfo.trackType = inTrackInfo->trackType;
		outTrackInfo.trackOrderIndex = inTrackInfo->trackOrderIndex;
		return AudioTrackInfoFromSdk(inTrackInfo->track, outTrackInfo.track);
	}

	void FullAudioTracksInfoFromSdk(SDK_SESSION_AUDIO_TRACK_FULL_INFO** inTracks, u32 numTracks, std::vector<SESSION_AUDIO_TRACK_FULL_INFO_NEW>& outTracks)
	{
		outTracks.clear();

		if (inTracks != NULL)
		{
			for (u32 index = 0; index != numTracks; ++index)
			{
				SESSION_AUDIO_TRACK_FULL_INFO_NEW newTrack;
				if (FullAudioTrackInfoFromSdk(inTracks[index], newTrack))
				{
					outTracks.push_back(newTrack);
				}
			}
		}
	}

	void EntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outEntityInfo)
	{
		outEntityInfo.clear();

		if (inEntityInfo != NULL)
		{
			for (u32 index = 0; index != inNumEntities; ++index)
			{
				const SDK_SESSION_ENTITY_INFO* entity = inEntityInfo[index];
				if (entity == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_ENTITY_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_ENTITY_INFO>(sdkInfo_myCopy, entity);

				SESSION_ENTITY_INFO_NEW newEntity;
				newEntity.dwWaveformID = entity->dwWaveformID;
				newEntity.dwFileFormat = entity->dwFileFormat;
				newEntity.iCodecDelay = entity->iCodecDelay;
				newEntity.dwTrueLength = entity->dwTrueLength;
				newEntity.iEntityType = entity->iEntityType;
				newEntity.wszFilename = StringFromSdk(entity->szFilename);

				outEntityInfo.push_back(newEntity);
			}
		}
	}

	void BlockEnvelopesFromSdk(SDK_SESSION_BLOCK_ENVELOPE_INFO** inEnvelopes, u32 numEnvelopes, std::vector<SESSION_BLOCK_ENVELOPE_INFO_NEW>& outEnvelopes)
	{
		outEnvelopes.clear();

		if (inEnvelopes != NULL)
		{
			for (u32 index = 0; index != numEnvelopes; ++index)
			{
				const SDK_SESSION_BLOCK_ENVELOPE_INFO* envelope = inEnvelopes[index];
				if (envelope == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_BLOCK_ENVELOPE_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_BLOCK_ENVELOPE_INFO>(sdkInfo_myCopy, envelope);

				SESSION_BLOCK_ENVELOPE_INFO_NEW newEnvelope;
				newEnvelope.uiType = envelope->uiType;
				newEnvelope.bSplined = BoolFromSdk(envelope->bSplined);
				EnvelopePointsFromSdk(envelope->points, envelope->numPoints, newEnvelope.points);

				outEnvelopes.push_back(newEnvelope);
			}
		}
	}

	void CommonBlockInfoFromSdk(SDK_SESSION_COMMON_BLOCK_INFO** inBlocks, u32 numBlocks, std::vector<SESSION_COMMON_BLOCK_INFO_NEW>& outBlocks)
	{
		outBlocks.clear();

		if (inBlocks != NULL)
		{
			for (u32 index = 0; index != numBlocks; ++index)
			{
				const SDK_SESSION_COMMON_BLOCK_INFO* block = inBlocks[index];
				if (block == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_COMMON_BLOCK_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_COMMON_BLOCK_INFO>(sdkInfo_myCopy, block);

				SESSION_COMMON_BLOCK_INFO_NEW newBlock;
				newBlock.dwBlockID = block->dwBlockID;
				newBlock.dwEntityID = block->dwEntityID;
				newBlock.fLeftVol = block->fLeftVol;
				newBlock.fRightVol = block->fRightVol;
				newBlock.dwOffset = block->dwOffset;
				newBlock.dwSize = block->dwSize;
				newBlock.dwFileIndent = block->dwFileIndent;
				newBlock.dwFlags = block->dwFlags;
				newBlock.iTrack = block->iTrack;
				newBlock.iParentGroup = block->iParentGroup;
				newBlock.iPunchGeneration = block->iPunchGeneration;
				newBlock.iPreviousPunch = block->iPreviousPunch;
				newBlock.iNextPunch = block->iNextPunch;
				newBlock.iHue = block->iHue;
				newBlock.iParentBlock = block->iParentBlock;

				outBlocks.push_back(newBlock);
			}
		}
	}

	void WaveBlockInfoFromSdk(SDK_SESSION_WAVE_BLOCK_INFO** inBlocks, u32 numBlocks, std::vector<SESSION_WAVE_BLOCK_INFO_NEW>& outBlocks)
	{
		outBlocks.clear();

		if (inBlocks != NULL)
		{
			for (u32 index = 0; index != numBlocks; ++index)
			{
				const SDK_SESSION_WAVE_BLOCK_INFO* block = inBlocks[index];
				if (block == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_WAVE_BLOCK_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_WAVE_BLOCK_INFO>(sdkInfo_myCopy, block);

				SESSION_WAVE_BLOCK_INFO_NEW newBlock;
				newBlock.dwBlockID = block->dwBlockID;
				newBlock.iFadeInOffset = block->iFadeInOffset;
				newBlock.iFadeOutOffset = block->iFadeOutOffset;
				newBlock.iFadeInShapeVal = block->iFadeInShapeVal;
				newBlock.iFadeOutShapeVal = block->iFadeOutShapeVal;
				newBlock.iFadeInType = block->iFadeInType;
				newBlock.iFadeOutType = block->iFadeOutType;
				newBlock.hasCrossfadeLeft = BoolFromSdk(block->hasCrossfadeLeft);
				newBlock.hasCrossfadeRight = BoolFromSdk(block->hasCrossfadeRight);
				newBlock.iCrossfadeLeftLinkType = block->iCrossfadeLeftLinkType;
				newBlock.iCrossfadeRightLinkType = block->iCrossfadeRightLinkType;

				outBlocks.push_back(newBlock);
			}
		}
	}

	void VideoBlockInfoFromSdk(SDK_SESSION_VIDEO_BLOCK_INFO** inBlocks, u32 numBlocks, std::vector<SESSION_VIDEO_BLOCK_INFO_NEW>& outBlocks)
	{
		outBlocks.clear();

		if (inBlocks != NULL)
		{
			for (u32 index = 0; index != numBlocks; ++index)
			{
				const SDK_SESSION_VIDEO_BLOCK_INFO* block = inBlocks[index];
				if (block == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_VIDEO_BLOCK_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_VIDEO_BLOCK_INFO>(sdkInfo_myCopy, block);

				SESSION_VIDEO_BLOCK_INFO_NEW newBlock;
				newBlock.dwBlockID = block->dwBlockID;
				newBlock.dwFlags = block->dwFlags;
				newBlock.wszAudioFileName = StringFromSdk(block->szAudioFileName);

				outBlocks.push_back(newBlock);
			}
		}
	}

	bool BlockEnvelopeGroupFromSdk(const SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* inEnvelopeGroup, SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW& outEnvelopeGroup)
	{
		// First, check to see if we have any sdk information at all
		if (inEnvelopeGroup == NULL)
		{
			return false;
		}

		// Next, ensure backwards compatibility
		std::auto_ptr<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO> sdkInfo_myCopy;
		EnsureBackwardCompatible<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO>(sdkInfo_myCopy, inEnvelopeGroup);

		// Convert between the SDK struct and our common struct
		outEnvelopeGroup.dwID = inEnvelopeGroup->dwID;
		outEnvelopeGroup.rightMostSampleInEnvelope = inEnvelopeGroup->rightMostSampleInEnvelopes;
		BlockEnvelopesFromSdk(inEnvelopeGroup->envelopes, inEnvelopeGroup->numEnvelopes, outEnvelopeGroup.envelopes);

		return true;
	}

	void MultiChannelEncoderTrackFromSdk(SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO** inTracks, u32 numTracks, std::vector<SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW>& outTracks)
	{
		outTracks.clear();

		if (inTracks != NULL)
		{
			for (u32 index = 0; index != numTracks; ++index)
			{
				const SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO* track = inTracks[index];
				if (track == NULL)
				{
					continue;
				}

				// Ensure backwards compatibility
				std::auto_ptr<SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO> sdkInfo_myCopy;
				EnsureBackwardCompatible<SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO>(sdkInfo_myCopy, track);

				SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW newTrack;
				newTrack.bUseFlatLines = BoolFromSdk(track->bUseFlatLines);
				newTrack.fFlatFade = track->fFlatFade;
				newTrack.fFlatPan = track->fFlatPan;
				newTrack.iSubLevel = track->iSubLevel;
				newTrack.iCenterLevel = track->iCenterLevel;
				newTrack.iTrackVolume = track->iTrackVolume;
				newTrack.dwActiveSpeakers = track->dwActiveSpeakers;
				newTrack.iPanningMode = track->iPanningMode;
				newTrack.bFixedRouting = BoolFromSdk(track->bFixedRouting);
				newTrack.iTrack = track->iTrack;
				if (BlockEnvelopeGroupFromSdk(track->envelopeGroup, newTrack.envelopeGroup))
				{
					outTracks.push_back(newTrack);
				}
			}
		}
	}

}	// private namespace



COMPLETE_SESSION_INFO_NEW& SessionIterpretter_FromSDK::GetSessionDataFromSdk(FULL_SDK_SESSION_INFO* inSdkSessionInfo)
{
	m_sessionInfo.sessionMainInfo.first = MainSessionInfoFromSdk(inSdkSessionInfo->sessionMainInfo, m_sessionInfo.sessionMainInfo.second);
	m_sessionInfo.versionInfo.first = VersionInfoFromSdk(inSdkSessionInfo->versionInfo, m_sessionInfo.versionInfo.second);
	m_sessionInfo.notesInfo.first = NotesInfoFromSdk(inSdkSessionInfo->notesInfo, m_sessionInfo.notesInfo.second);
	m_sessionInfo.stateInfo.first = StateInfoFromSdk(inSdkSessionInfo->stateInfo, m_sessionInfo.stateInfo.second);
	m_sessionInfo.tempoInfo.first = TempoInfoFromSdk(inSdkSessionInfo->tempoInfo, m_sessionInfo.tempoInfo.second);
	m_sessionInfo.tracksInfo.first = TracksInfoFromSdk(inSdkSessionInfo->tracksInfo, m_sessionInfo.tracksInfo.second);
	m_sessionInfo.waveEntityInfo.first = WaveEntityInfoFromSdk(inSdkSessionInfo->waveEntityInfo, inSdkSessionInfo->numWaveEntities, m_sessionInfo.waveEntityInfo.second);
	m_sessionInfo.videoEntityInfo.first = VideoEntityInfoFromSdk(inSdkSessionInfo->videoEntityInfo, inSdkSessionInfo->numVideoEntities, m_sessionInfo.videoEntityInfo.second);
	m_sessionInfo.groupInfo.first = GroupInfoFromSdk(inSdkSessionInfo->groupInfo, inSdkSessionInfo->numGroups, m_sessionInfo.groupInfo.second);
	m_sessionInfo.blockInfo.first = BlockInfoFromSdk(inSdkSessionInfo->blockInfo, m_sessionInfo.blockInfo.second);
	m_sessionInfo.blockEnvelopes.first = EnvelopeInfoFromSdk(inSdkSessionInfo->blockEnvelopes, inSdkSessionInfo->numBlockEnvelopes, m_sessionInfo.blockEnvelopes.second);
	m_sessionInfo.loopInfo.first = LoopInfoFromSdk(inSdkSessionInfo->loopInfo, inSdkSessionInfo->numLoops, m_sessionInfo.loopInfo.second);
	m_sessionInfo.cueInfo.first = CueInfoFromSdk(inSdkSessionInfo->cueInfo, inSdkSessionInfo->numCues, m_sessionInfo.cueInfo.second);
	m_sessionInfo.metronomeInfo.first = MetronomeInfoFromSdk(inSdkSessionInfo->metronomeInfo, m_sessionInfo.metronomeInfo.second);
	m_sessionInfo.multiChannelEncoderInfo.first = MultiChannelEncoderInfoFromSdk(inSdkSessionInfo->multiChannelEncoderInfo, m_sessionInfo.multiChannelEncoderInfo.second);

	return m_sessionInfo;
}


#pragma pack(1)

struct SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW : public SDK_SESSION_ENVELOPE_POINT_INFO
{
	int x;
};

struct SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD
{
	u32	mySize;
	i64 dwAt;
};

#pragma pack()


void SessionIterpretter_FromSDK::test()
{
	// This is meant to test the forward and backward compatibility...

	SDK_SESSION_ENVELOPE_POINT_INFO* points = new SDK_SESSION_ENVELOPE_POINT_INFO[3];
	points[0].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO);
	points[0].dwAt = 50;
	points[0].value = 0.5;
	points[1].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO);
	points[1].dwAt = 100;
	points[1].value = 0.6;
	points[2].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO);
	points[2].dwAt = 150;
	points[2].value = 0.7;
	SDK_SESSION_ENVELOPE_POINT_INFO* pointListing[3];
	pointListing[0] = &points[0];
	pointListing[1] = &points[1];
	pointListing[2] = &points[2];

	SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW* points2 = new SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW[3];
	points2[0].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW);
	points2[0].dwAt = 350;
	points2[0].value = 0.1;
	points2[1].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW);
	points2[1].dwAt = 400;
	points2[1].value = 0.2;
	points2[2].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW);
	points2[2].dwAt = 450;
	points2[2].value = 0.3;
	SDK_SESSION_ENVELOPE_POINT_INFO_NEWNEW* pointListing2[3];
	pointListing2[0] = &points2[0];
	pointListing2[1] = &points2[1];
	pointListing2[2] = &points2[2];

	SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD* points3 = new SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD[3];
	points3[0].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD);
	points3[0].dwAt = 777;
	points3[1].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD);
	points3[1].dwAt = 888;
	points3[2].mySize = sizeof(SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD);
	points3[2].dwAt = 999;
	SDK_SESSION_ENVELOPE_POINT_INFO_OLDOLD* pointListing3[3];
	pointListing3[0] = &points3[0];
	pointListing3[1] = &points3[1];
	pointListing3[2] = &points3[2];


	std::vector<SESSION_ENVELOPE_POINT_INFO_NEW> envelope1;
	EnvelopePointsFromSdk(pointListing, 3, envelope1);
	std::vector<SESSION_ENVELOPE_POINT_INFO_NEW> envelope2;
	EnvelopePointsFromSdk((SDK_SESSION_ENVELOPE_POINT_INFO**)pointListing2, 3, envelope2);
	std::vector<SESSION_ENVELOPE_POINT_INFO_NEW> envelope3;
	EnvelopePointsFromSdk((SDK_SESSION_ENVELOPE_POINT_INFO**)pointListing3, 3, envelope3);

	int x = 1;
}


bool SessionIterpretter_FromSDK::MainSessionInfoFromSdk(SDK_SESSION_MAIN_INFO* inSdkMainSessionInfo, SESSION_MAIN_INFO_NEW& outMainSessionInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkMainSessionInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_MAIN_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_MAIN_INFO>(sdkInfo_myCopy, inSdkMainSessionInfo);

	// Convert between the SDK struct and our common struct
	outMainSessionInfo.iSampleRate = inSdkMainSessionInfo->iSampleRate;
	outMainSessionInfo.qiNumSamples = inSdkMainSessionInfo->qiNumSamples;
	outMainSessionInfo.fMasterVolume = inSdkMainSessionInfo->fMasterVolume;
	outMainSessionInfo.qiMainOffset = inSdkMainSessionInfo->qiMainOffset;
	outMainSessionInfo.bRecovery = BoolFromSdk(inSdkMainSessionInfo->bRecovery);
	outMainSessionInfo.iRulerFormat = inSdkMainSessionInfo->iRulerFormat;
	outMainSessionInfo.bPanningMode = inSdkMainSessionInfo->bPanningMode;
	outMainSessionInfo.fVolEnvRange = inSdkMainSessionInfo->fVolEnvRange;
	outMainSessionInfo.wszFilename = StringFromSdk(inSdkMainSessionInfo->szFilename);
	outMainSessionInfo.wszMaster = StringFromSdk(inSdkMainSessionInfo->szMaster);
	outMainSessionInfo.wszASIODriver = StringFromSdk(inSdkMainSessionInfo->szASIODriver);
	outMainSessionInfo.iTakeNumber = inSdkMainSessionInfo->iTakeNumber;
	outMainSessionInfo.iNumberOfBlocks = inSdkMainSessionInfo->iNumberOfBlocks;

	return true;
}

bool SessionIterpretter_FromSDK::VersionInfoFromSdk(SDK_SESSION_VERSION_INFO* inSdkVersionInfo, VERSION_INFO_NEW& outVersionInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkVersionInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_VERSION_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_VERSION_INFO>(sdkInfo_myCopy, inSdkVersionInfo);

	// Convert between the SDK struct and our common struct
	outVersionInfo.version = inSdkVersionInfo->version;
	outVersionInfo.buildNumber = inSdkVersionInfo->buildNumber;
	outVersionInfo.szVersionStringInfo = StringFromSdk(inSdkVersionInfo->szVersionStringInfo);

	return true;
}

bool SessionIterpretter_FromSDK::NotesInfoFromSdk(SDK_SESSION_NOTES_INFO* inSdkNotesInfo, std::wstring& outSessionNotesInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkNotesInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_NOTES_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_NOTES_INFO>(sdkInfo_myCopy, inSdkNotesInfo);

	// Convert between the SDK struct and our common struct
	outSessionNotesInfo = StringFromSdk(inSdkNotesInfo->notesInfo);

	return true;
}

bool SessionIterpretter_FromSDK::StateInfoFromSdk(SDK_SESSION_STATE_INFO* inSdkStateInfo, SESSION_STATE_INFO_NEW& outStateInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkStateInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_STATE_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_STATE_INFO>(sdkInfo_myCopy, inSdkStateInfo);

	// Convert between the SDK struct and our common struct
	outStateInfo.fLeftSample = inSdkStateInfo->fLeftSample;
	outStateInfo.fRightSample = inSdkStateInfo->fRightSample;
	outStateInfo.fLoTrack = inSdkStateInfo->fLoTrack;
	outStateInfo.fHiTrack = inSdkStateInfo->fHiTrack;
	outStateInfo.qiLoSample = inSdkStateInfo->qiLoSample;
	outStateInfo.qiHiSample = inSdkStateInfo->qiHiSample;
	outStateInfo.fNewLoTrack = inSdkStateInfo->fNewLoTrack;

	return true;
}

bool SessionIterpretter_FromSDK::TempoInfoFromSdk(SDK_SESSION_TEMPO_INFO* inSdkTempoInfo, SESSION_TEMPO_INFO_NEW& outTempoInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkTempoInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_TEMPO_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_TEMPO_INFO>(sdkInfo_myCopy, inSdkTempoInfo);

	// Convert between the SDK struct and our common struct
	outTempoInfo.fBeatsPerMinute = inSdkTempoInfo->fBeatsPerMinute;
	outTempoInfo.iBeatsPerBar = inSdkTempoInfo->iBeatsPerBar;
	outTempoInfo.iTicksPerQuarterNote = inSdkTempoInfo->iTicksPerQuarterNote;
	outTempoInfo.fBeatOffsetMs = inSdkTempoInfo->fBeatOffsetMs;
	outTempoInfo.iSessionBaseNote = inSdkTempoInfo->iSessionBaseNote;
	outTempoInfo.iNoteLen = inSdkTempoInfo->iNoteLen;

	return true;
}

bool SessionIterpretter_FromSDK::TracksInfoFromSdk(SDK_SESSION_ALL_TRACKS_INFO* inSdkTracksInfo, SESSION_ALL_TRACKS_INFO_NEW& outTracksInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkTracksInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_ALL_TRACKS_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_ALL_TRACKS_INFO>(sdkInfo_myCopy, inSdkTracksInfo);

	// Convert between the SDK struct and our common struct
	outTracksInfo.nextTrackID = inSdkTracksInfo->nextTrackID;
	FullAudioTrackInfoFromSdk(inSdkTracksInfo->masterTrack, outTracksInfo.masterTrack);
	FullAudioTracksInfoFromSdk(inSdkTracksInfo->audioTracks, inSdkTracksInfo->numAudioTracks, outTracksInfo.audioTracks);
	FullCommonTracksInfoFromSdk(inSdkTracksInfo->nonAudioTracks, inSdkTracksInfo->numNonAudioTracks, outTracksInfo.nonAudioTracks);

	return true;
}

bool SessionIterpretter_FromSDK::WaveEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkWaveEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outWaveEntityInfo)
{
	outWaveEntityInfo.clear();
	EntityInfoFromSdk(inSdkWaveEntityInfo, inNumEntities, outWaveEntityInfo);
	return !outWaveEntityInfo.empty();
}

bool SessionIterpretter_FromSDK::MidiEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkMidiEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outMidiEntityInfo)
{
	outMidiEntityInfo.clear();
	EntityInfoFromSdk(inSdkMidiEntityInfo, inNumEntities, outMidiEntityInfo);
	return !outMidiEntityInfo.empty();
}

bool SessionIterpretter_FromSDK::VideoEntityInfoFromSdk(SDK_SESSION_ENTITY_INFO** inSdkVideoEntityInfo, unsigned int inNumEntities, std::vector<SESSION_ENTITY_INFO_NEW>& outVideoEntityInfo)
{
	outVideoEntityInfo.clear();
	EntityInfoFromSdk(inSdkVideoEntityInfo, inNumEntities, outVideoEntityInfo);
	return !outVideoEntityInfo.empty();
}

bool SessionIterpretter_FromSDK::GroupInfoFromSdk(SDK_SESSION_GROUP_INFO** inSdkGroupInfo, unsigned int inNumGroups, std::vector<SESSION_GROUP_INFO_NEW>& outGroupInfo)
{
	outGroupInfo.clear();

	if (inSdkGroupInfo != NULL)
	{
		for (u32 index = 0; index != inNumGroups; ++index)
		{
			const SDK_SESSION_GROUP_INFO* group = inSdkGroupInfo[index];
			if (group == NULL)
			{
				continue;
			}

			// Ensure backwards compatibility
			std::auto_ptr<SDK_SESSION_GROUP_INFO> sdkInfo_myCopy;
			EnsureBackwardCompatible<SDK_SESSION_GROUP_INFO>(sdkInfo_myCopy, group);

			SESSION_GROUP_INFO_NEW newGroup;
			newGroup.iIndex = group->iIndex;
			newGroup.iColor = group->iColor;

			outGroupInfo.push_back(newGroup);
		}
	}

	return !outGroupInfo.empty();
}

bool SessionIterpretter_FromSDK::BlockInfoFromSdk(SDK_SESSION_BLOCK_INFO* inSdkBlockInfo, SESSION_BLOCK_INFO_NEW& outBlockInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkBlockInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_BLOCK_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_BLOCK_INFO>(sdkInfo_myCopy, inSdkBlockInfo);

	// Convert between the SDK struct and our common struct
	CommonBlockInfoFromSdk(inSdkBlockInfo->commonBlockInfo, inSdkBlockInfo->numCommonBlockInfo, outBlockInfo.commonBlockInfo);
	WaveBlockInfoFromSdk(inSdkBlockInfo->waveBlockInfo, inSdkBlockInfo->numWaveBlockInfo, outBlockInfo.waveBlockInfo);
	VideoBlockInfoFromSdk(inSdkBlockInfo->videoBlockInfo, inSdkBlockInfo->numVideoBlockInfo, outBlockInfo.videoBlockInfo);

	return true;
}

bool SessionIterpretter_FromSDK::EnvelopeInfoFromSdk(SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO** inSdkEnvelopeInfo, unsigned int inNumEnvelopeGroups, std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& outEnvelopeInfo)
{
	outEnvelopeInfo.clear();

	if (inSdkEnvelopeInfo != NULL)
	{
		for (u32 index = 0; index != inNumEnvelopeGroups; ++index)
		{
			const SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* envelope = inSdkEnvelopeInfo[index];
			if (envelope == NULL)
			{
				continue;
			}

			// Ensure backwards compatibility
			std::auto_ptr<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO> sdkInfo_myCopy;
			EnsureBackwardCompatible<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO>(sdkInfo_myCopy, envelope);

			SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW newEnvelope;
			newEnvelope.dwID = envelope->dwID;
			newEnvelope.rightMostSampleInEnvelope = envelope->rightMostSampleInEnvelopes;
			BlockEnvelopesFromSdk(envelope->envelopes, envelope->numEnvelopes, newEnvelope.envelopes);

			outEnvelopeInfo.push_back(newEnvelope);
		}
	}

	return !outEnvelopeInfo.empty();
}

bool SessionIterpretter_FromSDK::LoopInfoFromSdk(SDK_SESSION_LOOP_INFO** inSdkLoopInfo, unsigned int inNumLoops, std::vector<SESSION_LOOP_INFO_NEW>& outLoopInfo)
{
	outLoopInfo.clear();

	if (inSdkLoopInfo != NULL)
	{
		for (u32 index = 0; index != inNumLoops; ++index)
		{
			const SDK_SESSION_LOOP_INFO* loop = inSdkLoopInfo[index];
			if (loop == NULL)
			{
				continue;
			}

			// Ensure backwards compatibility
			std::auto_ptr<SDK_SESSION_LOOP_INFO> sdkInfo_myCopy;
			EnsureBackwardCompatible<SDK_SESSION_LOOP_INFO>(sdkInfo_myCopy, loop);

			SESSION_LOOP_INFO_NEW newLoop;
			newLoop.dwID = loop->dwID;
			newLoop.fNumBeats = loop->fNumBeats;
			newLoop.fNativeBpm = loop->fNativeBpm;
			newLoop.iBaseNote = loop->iBaseNote;
			newLoop.bLoopSimple = BoolFromSdk(loop->bLoopSimple);
			newLoop.bLoopInSamples = BoolFromSdk(loop->bLoopInSamples);
			newLoop.dwLoopSamples = loop->dwLoopSamples;
			newLoop.bLoopInBeats = BoolFromSdk(loop->bLoopInBeats);
			newLoop.fLoopEveryNBeats = loop->fLoopEveryNBeats;
			newLoop.bFollowSessionTempo = BoolFromSdk(loop->bFollowSessionTempo);
			newLoop.fLoopTempo = loop->fLoopTempo;
			newLoop.bTempoMatchFixed = BoolFromSdk(loop->bTempoMatchFixed);
			newLoop.bTempoMatchStretch = BoolFromSdk(loop->bTempoMatchStretch);
			newLoop.bTempoMatchResample = BoolFromSdk(loop->bTempoMatchResample);
			newLoop.bTempoMatchBeatSplice = BoolFromSdk(loop->bTempoMatchBeatSplice);
			newLoop.iBeatSpliceRate = loop->iBeatSpliceRate;
			newLoop.bCrossfade = BoolFromSdk(loop->bCrossfade);
			newLoop.fSessionTempoUsed = loop->fSessionTempoUsed;
			newLoop.iStretchQuality = loop->iStretchQuality;
			newLoop.fStretchSplicesPerBeat = loop->fStretchSplicesPerBeat;
			newLoop.fStretchOverlapping = loop->fStretchOverlapping;
			newLoop.bBeatSpliceAutoFind = BoolFromSdk(loop->bBeatSpliceAutoFind);
			newLoop.fBeatSpliceDb = loop->fBeatSpliceDb;
			newLoop.fBeatSpliceMs = loop->fBeatSpliceMs;
			newLoop.dwOrigFileIndent = loop->dwOrigFileIndent;
			newLoop.fTranspose = loop->fTranspose;
			newLoop.bLoopOnce = BoolFromSdk(loop->bLoopOnce);

			outLoopInfo.push_back(newLoop);
		}
	}

	return !outLoopInfo.empty();
}

bool SessionIterpretter_FromSDK::CueInfoFromSdk(SDK_SESSION_CUE_INFO** inSdkCueInfo, unsigned int inNumCues, std::vector<SESSION_CUE_INFO_NEW>& outCueInfo)
{
	outCueInfo.clear();

	if (inSdkCueInfo != NULL)
	{
		for (u32 index = 0; index != inNumCues; ++index)
		{
			const SDK_SESSION_CUE_INFO* cue = inSdkCueInfo[index];
			if (cue == NULL)
			{
				continue;
			}

			// Ensure backwards compatibility
			std::auto_ptr<SDK_SESSION_CUE_INFO> sdkInfo_myCopy;
			EnsureBackwardCompatible<SDK_SESSION_CUE_INFO>(sdkInfo_myCopy, cue);

			SESSION_CUE_INFO_NEW newCue;
			newCue.dwType = cue->dwType;
			newCue.qiOffset = cue->qiOffset;
			newCue.qiLength = cue->qiLength;
			newCue.wszName = StringFromSdk(cue->wszName);
			newCue.wszDesc = StringFromSdk(cue->wszDesc);

			outCueInfo.push_back(newCue);
		}
	}

	return !outCueInfo.empty();
}

bool SessionIterpretter_FromSDK::MetronomeInfoFromSdk(SDK_SESSION_METRONOME_INFO* inSdkMetronomeInfo, SESSION_METRONOME_INFO_NEW& outMetronomeInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkMetronomeInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_METRONOME_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_METRONOME_INFO>(sdkInfo_myCopy, inSdkMetronomeInfo);

	// Convert between the SDK struct and our common struct
	outMetronomeInfo.iSampleRate = inSdkMetronomeInfo->iSampleRate;
	outMetronomeInfo.fBPM = inSdkMetronomeInfo->fBPM;
	outMetronomeInfo.fOffsetMs = inSdkMetronomeInfo->fOffsetMs;
	outMetronomeInfo.fVolume = inSdkMetronomeInfo->fVolume;
	outMetronomeInfo.iTickSpacing = inSdkMetronomeInfo->iTickSpacing;
	outMetronomeInfo.iOffset = inSdkMetronomeInfo->iOffset;
	outMetronomeInfo.iSet = inSdkMetronomeInfo->iSet;
	outMetronomeInfo.bEnable = BoolFromSdk(inSdkMetronomeInfo->bEnable);
	outMetronomeInfo.szDefaultTime = StringFromSdk(inSdkMetronomeInfo->szDefaultTime);
	outMetronomeInfo.szPattern = StringFromSdk(inSdkMetronomeInfo->szPattern);
	outMetronomeInfo.dwOutputPort = inSdkMetronomeInfo->dwOutputPort;
	outMetronomeInfo.iNoteLen = inSdkMetronomeInfo->iNoteLen;

	return true;
}

bool SessionIterpretter_FromSDK::MultiChannelEncoderInfoFromSdk(SDK_SESSION_MULTICHANNELENCODER_INFO* inSdkMultiChannelEncoderInfo, SESSION_MULTICHANNELENCODER_INFO_NEW& outMultiChannelEncoderInfo)
{
	// First, check to see if we have any sdk information at all
	if (inSdkMultiChannelEncoderInfo == NULL)
	{
		return false;
	}

	// Next, ensure backwards compatibility
	std::auto_ptr<SDK_SESSION_MULTICHANNELENCODER_INFO> sdkInfo_myCopy;
	EnsureBackwardCompatible<SDK_SESSION_MULTICHANNELENCODER_INFO>(sdkInfo_myCopy, inSdkMultiChannelEncoderInfo);

	// Convert between the SDK struct and our common struct
	outMultiChannelEncoderInfo.fMasterVolume = inSdkMultiChannelEncoderInfo->fMasterVolume;
	outMultiChannelEncoderInfo.sessionName = StringFromSdk(inSdkMultiChannelEncoderInfo->sessionName);
	MultiChannelEncoderTrackFromSdk(inSdkMultiChannelEncoderInfo->tracks, inSdkMultiChannelEncoderInfo->numTracks, outMultiChannelEncoderInfo.tracks);

	return true;
}


}	// namespace AuditionSessionSDK
