/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2006-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#define STRICT

#include "SessionSdkInterpretter.h"
#include <assert.h>
#include <Windows.h>


namespace AuditionSessionSDK
{


// The following classes (SdkAllocatedObject_Abstract, and both inherited template classes)
// help wrap pointers to anything inside a class that will, upon destruction, delete their 
// wrapped pointers.  The SdkAllocatedObject_Abstract common base allows us to take all these 
// different pointers and store them together in a single container for easier memory management.

// The concept is taken one step further with the MemoryDeallocator class.  The MemoryDeallocator class
// wraps a pointer to SdkAllocatedObject_Abstract, which in turn, wraps a pointer to anything.
// The MemoryDeallocator class is a copyable, ref-counted object so it can safely be used in
// STL containers, like a vector.  It acts much like a specialized shared pointer.  When the last
// copy is destroyed, it will delete the pointer to SdkAllocatedObject_Abstract, which in turn, 
// delete's whatever pointer it has.

class SdkAllocatedObject_Abstract
{
public:
	virtual ~SdkAllocatedObject_Abstract() {}
};


template<typename T>
class SdkAllocatedObject : public SdkAllocatedObject_Abstract
{
public:
	SdkAllocatedObject(T* inPointer) : mPointer(inPointer) {}
	virtual ~SdkAllocatedObject() { delete mPointer; }
	
private:
	T* mPointer;
};

template<typename T>
class SdkAllocatedList : public SdkAllocatedObject_Abstract
{
public:
	SdkAllocatedList(T* inPointer) : mPointer(inPointer) {}
	virtual ~SdkAllocatedList() { delete [] mPointer; }

private:
	T* mPointer;
};



MemoryDeallocator::MemoryDeallocator(SdkAllocatedObject_Abstract* inAllocatedObject)
	:
	mAllocatedObject(inAllocatedObject),
	mRefCount(new u32)
{
	*mRefCount = 1;
}

MemoryDeallocator::MemoryDeallocator(const MemoryDeallocator& inMemoryDeallocator)
	:
	mAllocatedObject(inMemoryDeallocator.mAllocatedObject),
	mRefCount(inMemoryDeallocator.mRefCount)
{
	AddRef();
}

MemoryDeallocator::~MemoryDeallocator()
{
	Release();
}

MemoryDeallocator& MemoryDeallocator::operator=(const MemoryDeallocator& inRhs)
{
	Release();
	mAllocatedObject = inRhs.mAllocatedObject;
	mRefCount = inRhs.mRefCount;
	AddRef();

	return *this;
}

void	MemoryDeallocator::AddRef()
{
	++*mRefCount;
}

void	MemoryDeallocator::Release()
{
	--*mRefCount;
	if (*mRefCount == 0)
	{
		delete mAllocatedObject;
		delete mRefCount;
	}
}



namespace 
{

	template<typename T>
	T* CreateSdkObject(AllocationList& ioAllocationList)
	{
		T* packagePointer = new T;
		SdkAllocatedObject<T>* allocatedObject = new SdkAllocatedObject<T>(packagePointer);
		ioAllocationList.push_back(allocatedObject);
		packagePointer->mySize = sizeof(T);

		return packagePointer;
	}

	template<typename T>
	T* CreateSdkList(const u32& inNumPoints, AllocationList& ioAllocationList)
	{
		T* packagePointer = new T[inNumPoints];
		SdkAllocatedList<T>* allocatedObject = new SdkAllocatedList<T>(packagePointer);
		ioAllocationList.push_back(allocatedObject);

		return packagePointer;
	}

	SDK_SESSION_STRING* StringForSdk(const std::wstring inString, AllocationList& ioAllocationList)
	{
		SDK_SESSION_STRING* sdkString = new SDK_SESSION_STRING;
		SdkAllocatedObject<SDK_SESSION_STRING>* allocatedObject = new SdkAllocatedObject<SDK_SESSION_STRING>(sdkString);
		ioAllocationList.push_back(allocatedObject);

		sdkString->stringSize = inString.size() + 1;
		sdkString->chars = CreateSdkList<c16>(sdkString->stringSize, ioAllocationList);

		std::wstring::const_iterator iter = inString.begin();
		const std::wstring::const_iterator endIter = inString.end();
		for (u32 index = 0; iter != endIter && index < sdkString->stringSize; ++iter, ++index)
		{
			sdkString->chars[index] = *iter;
		}

		// Null terminator
		sdkString->chars[index] = 0;
		++index;

		assert(iter == endIter && index == sdkString->stringSize);

		return sdkString;
	}

	inline b32 BoolForSdk(const bool& inBool)
	{
		return inBool ? 1 : 0;
	}

	SDK_SESSION_GUID GuidForSdk(const GUID& inGuid)
	{
		SDK_SESSION_GUID sdkGuid;
		sdkGuid.Data1 = inGuid.Data1;
		sdkGuid.Data2 = inGuid.Data2;
		sdkGuid.Data3 = inGuid.Data3;
		for (int index = 0; index < 8; ++index)
		{
			sdkGuid.Data4[index] = inGuid.Data4[index];
		}

		return sdkGuid;
	}

	SDK_SESSION_ENVELOPE_POINT_INFO** EnvelopePointsForSdk(const std::vector<SESSION_ENVELOPE_POINT_INFO_NEW>& inPoints, u32& outNumPoints, AllocationList& ioAllocationList)
	{
		outNumPoints = inPoints.size();

		if (outNumPoints == 0)
		{
			return 0;
		}

		SDK_SESSION_ENVELOPE_POINT_INFO** outPoints = CreateSdkList<SDK_SESSION_ENVELOPE_POINT_INFO*>(outNumPoints, ioAllocationList);

		std::vector<SESSION_ENVELOPE_POINT_INFO_NEW>::const_iterator iter = inPoints.begin();
		const std::vector<SESSION_ENVELOPE_POINT_INFO_NEW>::const_iterator endIter = inPoints.end();
		for (u32 index = 0; iter != endIter && index < outNumPoints; ++iter, ++index)
		{
			SDK_SESSION_ENVELOPE_POINT_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_ENVELOPE_POINT_INFO>(ioAllocationList);
			sdkInfo->value = iter->value;
			sdkInfo->dwAt = iter->dwAt;

			outPoints[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumPoints);

		return outPoints;
	}

	SDK_SESSION_ENV_INFO* EnvelopeForSdk(const SESSION_ENV_INFO_NEW& inEnvelope, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_ENV_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_ENV_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->uiType = inEnvelope.uiType;
		sdkInfo->bSplined = BoolForSdk(inEnvelope.bSplined);
		sdkInfo->szName = StringForSdk(inEnvelope.szName, ioAllocationList);
		sdkInfo->dwFlags = inEnvelope.dwFlags;
		sdkInfo->LineColor = inEnvelope.LineColor;
		sdkInfo->dwSize = inEnvelope.dwSize;
		sdkInfo->bReadOnly = BoolForSdk(inEnvelope.bReadOnly);
		sdkInfo->points = EnvelopePointsForSdk(inEnvelope.points, sdkInfo->numPoints, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_ENV_INFO** EnvelopesForSdk(const std::vector<SESSION_ENV_INFO_NEW>& inEnvelopes, u32& outNumEnvelopes, AllocationList& ioAllocationList)
	{
		outNumEnvelopes = inEnvelopes.size();

		if (outNumEnvelopes == 0)
		{
			return 0;
		}

		SDK_SESSION_ENV_INFO** outPoints = CreateSdkList<SDK_SESSION_ENV_INFO*>(outNumEnvelopes, ioAllocationList);

		std::vector<SESSION_ENV_INFO_NEW>::const_iterator iter = inEnvelopes.begin();
		const std::vector<SESSION_ENV_INFO_NEW>::const_iterator endIter = inEnvelopes.end();
		for (u32 index = 0; iter != endIter && index < outNumEnvelopes; ++iter, ++index)
		{
			outPoints[index] = EnvelopeForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumEnvelopes);

		return outPoints;
	}

	SDK_SESSION_AUDIO_PARAM* AudioParamForSdk(const SESSION_AUDIO_PARAM_NEW& inAudioParam, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_PARAM* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_PARAM>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->bIsAutomatable = BoolForSdk(inAudioParam.bIsAutomatable);
		sdkInfo->parameterValueForRendering = inAudioParam.parameterValueForRendering;
		sdkInfo->bSafeDuringWrite = BoolForSdk(inAudioParam.bSafeDuringWrite);
		sdkInfo->envelope = EnvelopeForSdk(inAudioParam.envelope, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_PARAM** AudioParamsForSdk(const std::vector<SESSION_AUDIO_PARAM_NEW>& inAudioParams, u32& outNumParams, AllocationList& ioAllocationList)
	{
		outNumParams = inAudioParams.size();

		if (outNumParams == 0)
		{
			return 0;
		}

		SDK_SESSION_AUDIO_PARAM** outParams = CreateSdkList<SDK_SESSION_AUDIO_PARAM*>(outNumParams, ioAllocationList);

		std::vector<SESSION_AUDIO_PARAM_NEW>::const_iterator iter = inAudioParams.begin();
		const std::vector<SESSION_AUDIO_PARAM_NEW>::const_iterator endIter = inAudioParams.end();
		for (u32 index = 0; iter != endIter && index < outNumParams; ++iter, ++index)
		{
			outParams[index] = AudioParamForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumParams);

		return outParams;
	}

	SDK_SESSION_AUDIO_COMPONENT* AudioComponentForSdk(const SESSION_AUDIO_COMPONENT_NEW& inAudioComponent, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_COMPONENT* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_COMPONENT>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->componentType = inAudioComponent.componentType;
		sdkInfo->initialDelay = inAudioComponent.initialDelay;
		sdkInfo->isBypass = BoolForSdk(inAudioComponent.isBypass);
		sdkInfo->isSerial = BoolForSdk(inAudioComponent.isSerial);
		sdkInfo->audioParams = AudioParamsForSdk(inAudioComponent.audioParams, sdkInfo->numParams, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_OUTPUT_COMPONENT* AudioOutputComponentForSdk(const SESSION_AUDIO_OUTPUT_COMPONENT_NEW& inAudioOutputComponent, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_OUTPUT_COMPONENT* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_OUTPUT_COMPONENT>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->componentType = inAudioOutputComponent.componentType;
		sdkInfo->initialDelay = inAudioOutputComponent.initialDelay;
		sdkInfo->isBypass = BoolForSdk(inAudioOutputComponent.isBypass);
		sdkInfo->audioParams = AudioParamsForSdk(inAudioOutputComponent.audioParams, sdkInfo->numParams, ioAllocationList);
		sdkInfo->pannerMode = inAudioOutputComponent.pannerMode;
		sdkInfo->isSumDown = BoolForSdk(inAudioOutputComponent.isSumDown);
		sdkInfo->outputChannels = inAudioOutputComponent.outputChannels;
		sdkInfo->busID = inAudioOutputComponent.busID;

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_EFFECT_COMPONENT* AudioEffectComponentForSdk(const SESSION_AUDIO_EFFECT_COMPONENT_NEW& inAudioComponent, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_EFFECT_COMPONENT* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_EFFECT_COMPONENT>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->componentType = inAudioComponent.componentType;
		sdkInfo->initialDelay = inAudioComponent.initialDelay;
		sdkInfo->isBypass = BoolForSdk(inAudioComponent.isBypass);
		sdkInfo->audioParams = AudioParamsForSdk(inAudioComponent.audioParams, sdkInfo->numParams, ioAllocationList);
		sdkInfo->guid = GuidForSdk(inAudioComponent.guid);
		sdkInfo->name = StringForSdk(inAudioComponent.name, ioAllocationList);
		sdkInfo->groupName = StringForSdk(inAudioComponent.groupName, ioAllocationList);
		sdkInfo->effectParamsString = StringForSdk(inAudioComponent.effectParamsString, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_EFFECT_COMPONENT** AudioEffectComponentsForSdk(const std::vector<SESSION_AUDIO_EFFECT_COMPONENT_NEW>& inEffects, u32& outNumEffects, AllocationList& ioAllocationList)
	{
		outNumEffects = inEffects.size();

		if (outNumEffects == 0)
		{
			return 0;
		}

		SDK_SESSION_AUDIO_EFFECT_COMPONENT** outEffects = CreateSdkList<SDK_SESSION_AUDIO_EFFECT_COMPONENT*>(outNumEffects, ioAllocationList);

		std::vector<SESSION_AUDIO_EFFECT_COMPONENT_NEW>::const_iterator iter = inEffects.begin();
		const std::vector<SESSION_AUDIO_EFFECT_COMPONENT_NEW>::const_iterator endIter = inEffects.end();
		for (u32 index = 0; iter != endIter && index < outNumEffects; ++iter, ++index)
		{
			outEffects[index] = AudioEffectComponentForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumEffects);

		return outEffects;
	}

	SDK_SESSION_AUDIO_SEND_COMPONENT* AudioSendComponentForSdk(const SESSION_AUDIO_SEND_COMPONENT_NEW& inAudioComponent, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_SEND_COMPONENT* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_SEND_COMPONENT>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->componentType = inAudioComponent.componentType;
		sdkInfo->initialDelay = inAudioComponent.initialDelay;
		sdkInfo->isBypass = BoolForSdk(inAudioComponent.isBypass);
		sdkInfo->audioParams = AudioParamsForSdk(inAudioComponent.audioParams, sdkInfo->numParams, ioAllocationList);
		sdkInfo->pannerMode = inAudioComponent.pannerMode;
		sdkInfo->isSumDown = BoolForSdk(inAudioComponent.isSumDown);
		sdkInfo->numOutputChannels = inAudioComponent.numOutputChannels;
		sdkInfo->busID = inAudioComponent.busID;
		sdkInfo->isPre = BoolForSdk(inAudioComponent.isPre);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_SEND_COMPONENT** AudioSendComponentsForSdk(const std::vector<SESSION_AUDIO_SEND_COMPONENT_NEW>& inSends, u32& outNumSends, AllocationList& ioAllocationList)
	{
		outNumSends = inSends.size();

		if (outNumSends == 0)
		{
			return 0;
		}

		SDK_SESSION_AUDIO_SEND_COMPONENT** outSends = CreateSdkList<SDK_SESSION_AUDIO_SEND_COMPONENT*>(outNumSends, ioAllocationList);

		std::vector<SESSION_AUDIO_SEND_COMPONENT_NEW>::const_iterator iter = inSends.begin();
		const std::vector<SESSION_AUDIO_SEND_COMPONENT_NEW>::const_iterator endIter = inSends.end();
		for (u32 index = 0; iter != endIter && index < outNumSends; ++iter, ++index)
		{
			outSends[index] = AudioSendComponentForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumSends);

		return outSends;
	}

	SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP* AudioEffectComponentGroupForSdk(const SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW& inAudioComponent, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_EFFECT_COMPONENT_GROUP>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->componentType = inAudioComponent.componentType;
		sdkInfo->initialDelay = inAudioComponent.initialDelay;
		sdkInfo->isBypass = BoolForSdk(inAudioComponent.isBypass);
		sdkInfo->isSerial = BoolForSdk(inAudioComponent.isSerial);
		sdkInfo->audioParams = AudioParamsForSdk(inAudioComponent.audioParams, sdkInfo->numParams, ioAllocationList);
		sdkInfo->inputGain= AudioComponentForSdk(inAudioComponent.inputGain, ioAllocationList);
		sdkInfo->outputGain = AudioComponentForSdk(inAudioComponent.outputGain, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO** TrackAutomationLanesForSdk(const std::vector<SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW>& inAutomationLanes, u32& outNumAutomationLanes, AllocationList& ioAllocationList)
	{
		outNumAutomationLanes = inAutomationLanes.size();

		if (outNumAutomationLanes == 0)
		{
			return 0;
		}

		SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO** outAutomationLanes = CreateSdkList<SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO*>(outNumAutomationLanes, ioAllocationList);

		std::vector<SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW>::const_iterator iter = inAutomationLanes.begin();
		const std::vector<SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW>::const_iterator endIter = inAutomationLanes.end();
		for (u32 index = 0; iter != endIter && index < outNumAutomationLanes; ++iter, ++index)
		{
			SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO>(ioAllocationList);
			sdkInfo->audioComponentType = iter->audioComponentType;
			sdkInfo->guidString = StringForSdk(iter->guidString, ioAllocationList);
			sdkInfo->componentName = StringForSdk(iter->componentName, ioAllocationList);
			sdkInfo->parameterIndex = iter->parameterIndex;
			sdkInfo->laneUIHeight = iter->laneUIHeight;

			outAutomationLanes[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumAutomationLanes);

		return outAutomationLanes;
	}

	SDK_SESSION_COMMON_TRACK_INFO* CommonTrackInfoForSdk(const SESSION_COMMON_TRACK_INFO_NEW& inTrackInfo, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_COMMON_TRACK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_COMMON_TRACK_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->id = inTrackInfo.id;
		sdkInfo->dwFlags = inTrackInfo.dwFlags;
		sdkInfo->szTitle = StringForSdk(inTrackInfo.szTitle, ioAllocationList);
		sdkInfo->dwTrackHeight = inTrackInfo.dwTrackHeight;

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_TRACK_INFO* AudioTrackInfoForSdk(const SESSION_AUDIO_TRACK_INFO_NEW& inTrackInfo, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_TRACK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_TRACK_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->baseTrackInfo = CommonTrackInfoForSdk(inTrackInfo.baseTrackInfo, ioAllocationList);
		sdkInfo->trackType = inTrackInfo.trackType;
		sdkInfo->busID = inTrackInfo.busID;
		sdkInfo->automationMode = inTrackInfo.automationMode;
		sdkInfo->inputDriver = StringForSdk(inTrackInfo.inputDriver, ioAllocationList);
		sdkInfo->inputPort = inTrackInfo.inputPort;
		sdkInfo->inputPortName = StringForSdk(inTrackInfo.inputPortName, ioAllocationList);
		sdkInfo->outputDriver = StringForSdk(inTrackInfo.outputDriver, ioAllocationList);
		sdkInfo->outputPort = inTrackInfo.outputPort;
		sdkInfo->outputPortName = StringForSdk(inTrackInfo.outputPortName, ioAllocationList);
		sdkInfo->isPreMeter = BoolForSdk(inTrackInfo.isPreMeter);
		sdkInfo->isPreEffect = BoolForSdk(inTrackInfo.isPreEffect);
		sdkInfo->isFreeze = BoolForSdk(inTrackInfo.isFreeze);
		sdkInfo->isTrackEQUI = BoolForSdk(inTrackInfo.isTrackEQUI);
		sdkInfo->isEffectsRackUI = BoolForSdk(inTrackInfo.isEffectsRackUI);
		sdkInfo->inputComponent = AudioComponentForSdk(inTrackInfo.inputComponent, ioAllocationList);
		sdkInfo->eqComponent = AudioComponentForSdk(inTrackInfo.eqComponent, ioAllocationList);
		sdkInfo->faderComponent = AudioComponentForSdk(inTrackInfo.faderComponent, ioAllocationList);
		sdkInfo->muteComponent = AudioComponentForSdk(inTrackInfo.muteComponent, ioAllocationList);
		sdkInfo->outputComponent = AudioOutputComponentForSdk(inTrackInfo.outputComponent, ioAllocationList);
		sdkInfo->effectComponentGroup = AudioEffectComponentGroupForSdk(inTrackInfo.effectComponentGroup, ioAllocationList);
		sdkInfo->sendComponents = AudioSendComponentsForSdk(inTrackInfo.sendComponents, sdkInfo->numSends, ioAllocationList);
		sdkInfo->effectComponents = AudioEffectComponentsForSdk(inTrackInfo.effectComponents, sdkInfo->numEffects, ioAllocationList);

		sdkInfo->numEffectSlots = inTrackInfo.hasEffectInSlot.size();
		if (0 == sdkInfo->numEffectSlots)
		{
			sdkInfo->hasEffectInSlot = 0;
		}
		else
		{
			sdkInfo->hasEffectInSlot = CreateSdkList<b32>(sdkInfo->numEffectSlots, ioAllocationList);

			std::vector<bool>::const_iterator iter = inTrackInfo.hasEffectInSlot.begin();
			const std::vector<bool>::const_iterator endIter = inTrackInfo.hasEffectInSlot.end();
			for (u32 index = 0; iter != endIter && index < sdkInfo->numEffectSlots; ++iter, ++index)
			{
				sdkInfo->hasEffectInSlot[index] = BoolForSdk(*iter);
			}
			assert(iter == endIter && index == sdkInfo->numEffectSlots);
		}

		sdkInfo->isShowingAutomationLanes = BoolForSdk(inTrackInfo.isShowingAutomationLanes);
		sdkInfo->trackHeight = inTrackInfo.trackHeight;

		sdkInfo->automationLanes = TrackAutomationLanesForSdk(inTrackInfo.automationLanes, sdkInfo->numAutomationLanes, ioAllocationList);

		sdkInfo->midiHostEffect = AudioEffectComponentForSdk(inTrackInfo.midiHostEffect, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_COMMON_TRACK_FULL_INFO* FullCommonTrackInfoForSdk(const SESSION_COMMON_TRACK_FULL_INFO_NEW& inTrackInfo, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_COMMON_TRACK_FULL_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_COMMON_TRACK_FULL_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->trackType = inTrackInfo.trackType;
		sdkInfo->trackOrderIndex = inTrackInfo.trackOrderIndex;
		sdkInfo->track = CommonTrackInfoForSdk(inTrackInfo.track, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_COMMON_TRACK_FULL_INFO** FullCommonTracksInfoForSdk(const std::vector<SESSION_COMMON_TRACK_FULL_INFO_NEW>& inTracks, u32& outNumTracks, AllocationList& ioAllocationList)
	{
		outNumTracks = inTracks.size();

		if (outNumTracks == 0)
		{
			return 0;
		}

		SDK_SESSION_COMMON_TRACK_FULL_INFO** outTracks = CreateSdkList<SDK_SESSION_COMMON_TRACK_FULL_INFO*>(outNumTracks, ioAllocationList);

		std::vector<SESSION_COMMON_TRACK_FULL_INFO_NEW>::const_iterator iter = inTracks.begin();
		const std::vector<SESSION_COMMON_TRACK_FULL_INFO_NEW>::const_iterator endIter = inTracks.end();
		for (u32 index = 0; iter != endIter && index < outNumTracks; ++iter, ++index)
		{
			outTracks[index] = FullCommonTrackInfoForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumTracks);

		return outTracks;
	}

	SDK_SESSION_AUDIO_TRACK_FULL_INFO* FullAudioTrackInfoForSdk(const SESSION_AUDIO_TRACK_FULL_INFO_NEW& inTrackInfo, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_AUDIO_TRACK_FULL_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_AUDIO_TRACK_FULL_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->trackType = inTrackInfo.trackType;
		sdkInfo->trackOrderIndex = inTrackInfo.trackOrderIndex;
		sdkInfo->track = AudioTrackInfoForSdk(inTrackInfo.track, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_AUDIO_TRACK_FULL_INFO** FullAudioTracksInfoForSdk(const std::vector<SESSION_AUDIO_TRACK_FULL_INFO_NEW>& inTracks, u32& outNumTracks, AllocationList& ioAllocationList)
	{
		outNumTracks = inTracks.size();

		if (outNumTracks == 0)
		{
			return 0;
		}

		SDK_SESSION_AUDIO_TRACK_FULL_INFO** outTracks = CreateSdkList<SDK_SESSION_AUDIO_TRACK_FULL_INFO*>(outNumTracks, ioAllocationList);

		std::vector<SESSION_AUDIO_TRACK_FULL_INFO_NEW>::const_iterator iter = inTracks.begin();
		const std::vector<SESSION_AUDIO_TRACK_FULL_INFO_NEW>::const_iterator endIter = inTracks.end();
		for (u32 index = 0; iter != endIter && index < outNumTracks; ++iter, ++index)
		{
			outTracks[index] = FullAudioTrackInfoForSdk(*iter, ioAllocationList);
		}
		assert(iter == endIter && index == outNumTracks);

		return outTracks;
	}

	SDK_SESSION_ENTITY_INFO** EntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inEntityInfo, u32& outNumEntities, AllocationList& ioAllocationList)
	{
		outNumEntities = inEntityInfo.size();

		if (outNumEntities == 0)
		{
			return 0;
		}

		SDK_SESSION_ENTITY_INFO** outTracks = CreateSdkList<SDK_SESSION_ENTITY_INFO*>(outNumEntities, ioAllocationList);

		std::vector<SESSION_ENTITY_INFO_NEW>::const_iterator iter = inEntityInfo.begin();
		const std::vector<SESSION_ENTITY_INFO_NEW>::const_iterator endIter = inEntityInfo.end();
		for (u32 index = 0; iter != endIter && index < outNumEntities; ++iter, ++index)
		{
			SDK_SESSION_ENTITY_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_ENTITY_INFO>(ioAllocationList);
			sdkInfo->dwWaveformID = iter->dwWaveformID;
			sdkInfo->dwFileFormat = iter->dwFileFormat;
			sdkInfo->iCodecDelay = iter->iCodecDelay;
			sdkInfo->dwTrueLength = iter->dwTrueLength;
			sdkInfo->iEntityType = iter->iEntityType;
			sdkInfo->szFilename = StringForSdk(iter->wszFilename, ioAllocationList);

			outTracks[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumEntities);

		return outTracks;
	}

	SDK_SESSION_BLOCK_ENVELOPE_INFO** BlockEnvelopesForSdk(const std::vector<SESSION_BLOCK_ENVELOPE_INFO_NEW>& inEnvelopes, u32& outNumEnvelopes, AllocationList& ioAllocationList)
	{
		outNumEnvelopes = inEnvelopes.size();

		if (outNumEnvelopes == 0)
		{
			return 0;
		}

		SDK_SESSION_BLOCK_ENVELOPE_INFO** outEnvelopes = CreateSdkList<SDK_SESSION_BLOCK_ENVELOPE_INFO*>(outNumEnvelopes, ioAllocationList);

		std::vector<SESSION_BLOCK_ENVELOPE_INFO_NEW>::const_iterator iter = inEnvelopes.begin();
		const std::vector<SESSION_BLOCK_ENVELOPE_INFO_NEW>::const_iterator endIter = inEnvelopes.end();
		for (u32 index = 0; iter != endIter && index < outNumEnvelopes; ++iter, ++index)
		{
			SDK_SESSION_BLOCK_ENVELOPE_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_BLOCK_ENVELOPE_INFO>(ioAllocationList);
			sdkInfo->uiType = iter->uiType;
			sdkInfo->bSplined = BoolForSdk(iter->bSplined);
			sdkInfo->points = EnvelopePointsForSdk(iter->points, sdkInfo->numPoints, ioAllocationList);

			outEnvelopes[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumEnvelopes);

		return outEnvelopes;
	}

	SDK_SESSION_COMMON_BLOCK_INFO** CommonBlockInfoForSdk(const std::vector<SESSION_COMMON_BLOCK_INFO_NEW>& inBlocks, u32& outNumBlocks, AllocationList& ioAllocationList)
	{
		outNumBlocks = inBlocks.size();

		if (outNumBlocks == 0)
		{
			return 0;
		}

		SDK_SESSION_COMMON_BLOCK_INFO** outBlocks = CreateSdkList<SDK_SESSION_COMMON_BLOCK_INFO*>(outNumBlocks, ioAllocationList);

		std::vector<SESSION_COMMON_BLOCK_INFO_NEW>::const_iterator iter = inBlocks.begin();
		const std::vector<SESSION_COMMON_BLOCK_INFO_NEW>::const_iterator endIter = inBlocks.end();
		for (u32 index = 0; iter != endIter && index < outNumBlocks; ++iter, ++index)
		{
			SDK_SESSION_COMMON_BLOCK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_COMMON_BLOCK_INFO>(ioAllocationList);
			sdkInfo->dwBlockID = iter->dwBlockID;
			sdkInfo->dwEntityID = iter->dwEntityID;
			sdkInfo->fLeftVol = iter->fLeftVol;
			sdkInfo->fRightVol = iter->fRightVol;
			sdkInfo->dwOffset = iter->dwOffset;
			sdkInfo->dwSize = iter->dwSize;
			sdkInfo->dwFileIndent = iter->dwFileIndent;
			sdkInfo->dwFlags = iter->dwFlags;
			sdkInfo->iTrack = iter->iTrack;
			sdkInfo->iParentGroup = iter->iParentGroup;
			sdkInfo->iPunchGeneration = iter->iPunchGeneration;
			sdkInfo->iPreviousPunch = iter->iPreviousPunch;
			sdkInfo->iNextPunch = iter->iNextPunch;
			sdkInfo->iHue = iter->iHue;
			sdkInfo->iParentBlock = iter->iParentBlock;

			outBlocks[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumBlocks);

		return outBlocks;
	}

	SDK_SESSION_WAVE_BLOCK_INFO** WaveBlockInfoForSdk(const std::vector<SESSION_WAVE_BLOCK_INFO_NEW>& inBlocks, u32& outNumBlocks, AllocationList& ioAllocationList)
	{
		outNumBlocks = inBlocks.size();

		if (outNumBlocks == 0)
		{
			return 0;
		}

		SDK_SESSION_WAVE_BLOCK_INFO** outBlocks = CreateSdkList<SDK_SESSION_WAVE_BLOCK_INFO*>(outNumBlocks, ioAllocationList);

		std::vector<SESSION_WAVE_BLOCK_INFO_NEW>::const_iterator iter = inBlocks.begin();
		const std::vector<SESSION_WAVE_BLOCK_INFO_NEW>::const_iterator endIter = inBlocks.end();
		for (u32 index = 0; iter != endIter && index < outNumBlocks; ++iter, ++index)
		{
			SDK_SESSION_WAVE_BLOCK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_WAVE_BLOCK_INFO>(ioAllocationList);
			sdkInfo->dwBlockID = iter->dwBlockID;
			sdkInfo->iFadeInOffset = iter->iFadeInOffset;
			sdkInfo->iFadeOutOffset = iter->iFadeOutOffset;
			sdkInfo->iFadeInShapeVal = iter->iFadeInShapeVal;
			sdkInfo->iFadeOutShapeVal = iter->iFadeOutShapeVal;
			sdkInfo->iFadeInType = iter->iFadeInType;
			sdkInfo->iFadeOutType = iter->iFadeOutType;
			sdkInfo->hasCrossfadeLeft = BoolForSdk(iter->hasCrossfadeLeft);
			sdkInfo->hasCrossfadeRight = BoolForSdk(iter->hasCrossfadeRight);
			sdkInfo->iCrossfadeLeftLinkType = iter->iCrossfadeLeftLinkType;
			sdkInfo->iCrossfadeRightLinkType = iter->iCrossfadeRightLinkType;

			outBlocks[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumBlocks);

		return outBlocks;
	}

	SDK_SESSION_VIDEO_BLOCK_INFO** VideoBlockInfoForSdk(const std::vector<SESSION_VIDEO_BLOCK_INFO_NEW>& inBlocks, u32& outNumBlocks, AllocationList& ioAllocationList)
	{
		outNumBlocks = inBlocks.size();

		if (outNumBlocks == 0)
		{
			return 0;
		}

		SDK_SESSION_VIDEO_BLOCK_INFO** outBlocks = CreateSdkList<SDK_SESSION_VIDEO_BLOCK_INFO*>(outNumBlocks, ioAllocationList);

		std::vector<SESSION_VIDEO_BLOCK_INFO_NEW>::const_iterator iter = inBlocks.begin();
		const std::vector<SESSION_VIDEO_BLOCK_INFO_NEW>::const_iterator endIter = inBlocks.end();
		for (u32 index = 0; iter != endIter && index < outNumBlocks; ++iter, ++index)
		{
			SDK_SESSION_VIDEO_BLOCK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_VIDEO_BLOCK_INFO>(ioAllocationList);
			sdkInfo->dwBlockID = iter->dwBlockID;
			sdkInfo->dwFlags = iter->dwFlags;
			sdkInfo->szAudioFileName = StringForSdk(iter->wszAudioFileName, ioAllocationList);

			outBlocks[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumBlocks);

		return outBlocks;
	}

	SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* BlockEnvelopeGroupForSdk(const SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW& inEnvelopeGroup, AllocationList& ioAllocationList)
	{
		// First, create the sdk object
		SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO>(ioAllocationList);

		// Convert between our common struct and the SDK struct
		sdkInfo->dwID = inEnvelopeGroup.dwID;
		sdkInfo->rightMostSampleInEnvelopes = inEnvelopeGroup.rightMostSampleInEnvelope;
		sdkInfo->envelopes = BlockEnvelopesForSdk(inEnvelopeGroup.envelopes, sdkInfo->numEnvelopes, ioAllocationList);

		return sdkInfo;
	}

	SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO** MultiChannelEncoderTrackForSdk(const std::vector<SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW>& inTracks, u32& outNumTracks, AllocationList& ioAllocationList)
	{
		outNumTracks = inTracks.size();

		if (outNumTracks == 0)
		{
			return 0;
		}

		SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO** outTracks = CreateSdkList<SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO*>(outNumTracks, ioAllocationList);

		std::vector<SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW>::const_iterator iter = inTracks.begin();
		const std::vector<SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW>::const_iterator endIter = inTracks.end();
		for (u32 index = 0; iter != endIter && index < outNumTracks; ++iter, ++index)
		{
			SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_MULTICHANNELENCODER_TRACK_INFO>(ioAllocationList);
			sdkInfo->bUseFlatLines = BoolForSdk(iter->bUseFlatLines);
			sdkInfo->fFlatFade = iter->fFlatFade;
			sdkInfo->fFlatPan = iter->fFlatPan;
			sdkInfo->iSubLevel = iter->iSubLevel;
			sdkInfo->iCenterLevel = iter->iCenterLevel;
			sdkInfo->iTrackVolume = iter->iTrackVolume;
			sdkInfo->dwActiveSpeakers = iter->dwActiveSpeakers;
			sdkInfo->iPanningMode = iter->iPanningMode;
			sdkInfo->bFixedRouting = BoolForSdk(iter->bFixedRouting);
			sdkInfo->iTrack = iter->iTrack;
			sdkInfo->envelopeGroup = BlockEnvelopeGroupForSdk(iter->envelopeGroup, ioAllocationList);

			outTracks[index] = sdkInfo;
		}
		assert(iter == endIter && index == outNumTracks);

		return outTracks;
	}

}	// private namespace


void SessionIterpretter_ToSDK::ClearAllData()
{
	mAllocationList.clear();

	m_sdkSessionInfo.mySize = sizeof(m_sdkSessionInfo);
	m_sdkSessionInfo.sessionMainInfo = NULL;
	m_sdkSessionInfo.versionInfo = NULL;
	m_sdkSessionInfo.notesInfo = NULL;
	m_sdkSessionInfo.stateInfo = NULL;
	m_sdkSessionInfo.tempoInfo = NULL;
	m_sdkSessionInfo.tracksInfo = NULL;
	m_sdkSessionInfo.waveEntityInfo = NULL;
	m_sdkSessionInfo.numWaveEntities = 0;
	m_sdkSessionInfo.videoEntityInfo = NULL;
	m_sdkSessionInfo.numVideoEntities = 0;
	m_sdkSessionInfo.groupInfo = NULL;
	m_sdkSessionInfo.numGroups = 0;
	m_sdkSessionInfo.blockInfo = NULL;
	m_sdkSessionInfo.blockEnvelopes = NULL;
	m_sdkSessionInfo.numBlockEnvelopes = 0;
	m_sdkSessionInfo.loopInfo = NULL;
	m_sdkSessionInfo.numLoops = 0;
	m_sdkSessionInfo.cueInfo = NULL;
	m_sdkSessionInfo.numCues = 0;
	m_sdkSessionInfo.metronomeInfo = NULL;
	m_sdkSessionInfo.multiChannelEncoderInfo = NULL;
}


FULL_SDK_SESSION_INFO* SessionIterpretter_ToSDK::FormatSessionDataForSdk(const COMPLETE_SESSION_INFO_NEW& inSessionInfo)
{
	ClearAllData();

	if (inSessionInfo.sessionMainInfo.first)
	{
		m_sdkSessionInfo.sessionMainInfo = MainSessionInfoForSdk(inSessionInfo.sessionMainInfo.second);
	}

	if (inSessionInfo.versionInfo.first)
	{
		m_sdkSessionInfo.versionInfo = VersionInfoForSdk(inSessionInfo.versionInfo.second);
	}

	if (inSessionInfo.notesInfo.first)
	{
		m_sdkSessionInfo.notesInfo = NotesInfoForSdk(inSessionInfo.notesInfo.second);
	}

	if (inSessionInfo.stateInfo.first)
	{
		m_sdkSessionInfo.stateInfo = StateInfoForSdk(inSessionInfo.stateInfo.second);
	}

	if (inSessionInfo.tempoInfo.first)
	{
		m_sdkSessionInfo.tempoInfo = TempoInfoForSdk(inSessionInfo.tempoInfo.second);
	}

	if (inSessionInfo.tracksInfo.first)
	{
		m_sdkSessionInfo.tracksInfo = TracksInfoForSdk(inSessionInfo.tracksInfo.second);
	}

	if (inSessionInfo.waveEntityInfo.first)
	{
		m_sdkSessionInfo.waveEntityInfo = WaveEntityInfoForSdk(inSessionInfo.waveEntityInfo.second, m_sdkSessionInfo.numWaveEntities);
	}

	if (inSessionInfo.videoEntityInfo.first)
	{
		m_sdkSessionInfo.videoEntityInfo = VideoEntityInfoForSdk(inSessionInfo.videoEntityInfo.second, m_sdkSessionInfo.numVideoEntities);
	}

	if (inSessionInfo.groupInfo.first)
	{
		m_sdkSessionInfo.groupInfo = GroupInfoForSdk(inSessionInfo.groupInfo.second, m_sdkSessionInfo.numGroups);
	}

	if (inSessionInfo.blockInfo.first)
	{
		m_sdkSessionInfo.blockInfo = BlockInfoForSdk(inSessionInfo.blockInfo.second);
	}

	if (inSessionInfo.blockEnvelopes.first)
	{
		m_sdkSessionInfo.blockEnvelopes = EnvelopeInfoForSdk(inSessionInfo.blockEnvelopes.second, m_sdkSessionInfo.numBlockEnvelopes);
	}

	if (inSessionInfo.loopInfo.first)
	{
		m_sdkSessionInfo.loopInfo = LoopInfoForSdk(inSessionInfo.loopInfo.second, m_sdkSessionInfo.numLoops);
	}

	if (inSessionInfo.cueInfo.first)
	{
		m_sdkSessionInfo.cueInfo = CueInfoForSdk(inSessionInfo.cueInfo.second, m_sdkSessionInfo.numCues);
	}

	if (inSessionInfo.metronomeInfo.first)
	{
		m_sdkSessionInfo.metronomeInfo = MetronomeInfoForSdk(inSessionInfo.metronomeInfo.second);
	}

	if (inSessionInfo.multiChannelEncoderInfo.first)
	{
		m_sdkSessionInfo.multiChannelEncoderInfo = MultiChannelEncoderInfoForSdk(inSessionInfo.multiChannelEncoderInfo.second);
	}

	return &m_sdkSessionInfo;
}


SDK_SESSION_MAIN_INFO* SessionIterpretter_ToSDK::MainSessionInfoForSdk(const SESSION_MAIN_INFO_NEW& inMainSessionInfo)
{
	// First, create the sdk object
	SDK_SESSION_MAIN_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_MAIN_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->iSampleRate = inMainSessionInfo.iSampleRate;
	sdkInfo->qiNumSamples = inMainSessionInfo.qiNumSamples;
	sdkInfo->fMasterVolume = inMainSessionInfo.fMasterVolume;
	sdkInfo->qiMainOffset = inMainSessionInfo.qiMainOffset;
	sdkInfo->bRecovery = BoolForSdk(inMainSessionInfo.bRecovery);
	sdkInfo->iRulerFormat = inMainSessionInfo.iRulerFormat;
	sdkInfo->bPanningMode = inMainSessionInfo.bPanningMode;
	sdkInfo->fVolEnvRange = inMainSessionInfo.fVolEnvRange;
	sdkInfo->szFilename = StringForSdk(inMainSessionInfo.wszFilename, mAllocationList);
	sdkInfo->szMaster = StringForSdk(inMainSessionInfo.wszMaster, mAllocationList);
	sdkInfo->szASIODriver = StringForSdk(inMainSessionInfo.wszASIODriver, mAllocationList);
	sdkInfo->iTakeNumber = inMainSessionInfo.iTakeNumber;
	sdkInfo->iNumberOfBlocks = inMainSessionInfo.iNumberOfBlocks;

	return sdkInfo;
}

SDK_SESSION_VERSION_INFO* SessionIterpretter_ToSDK::VersionInfoForSdk(const VERSION_INFO_NEW& inVersionInfo)
{
	// First, create the sdk object
	SDK_SESSION_VERSION_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_VERSION_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->version = inVersionInfo.version;
	sdkInfo->buildNumber = inVersionInfo.buildNumber;
	sdkInfo->szVersionStringInfo = StringForSdk(inVersionInfo.szVersionStringInfo, mAllocationList);

	return sdkInfo;
}

SDK_SESSION_NOTES_INFO* SessionIterpretter_ToSDK::NotesInfoForSdk(const std::wstring& inSessionNotesInfo)
{
	// First, create the sdk object
	SDK_SESSION_NOTES_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_NOTES_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->notesInfo = StringForSdk(inSessionNotesInfo, mAllocationList);

	return sdkInfo;
}

SDK_SESSION_STATE_INFO* SessionIterpretter_ToSDK::StateInfoForSdk(const SESSION_STATE_INFO_NEW& inStateInfo)
{
	// First, create the sdk object
	SDK_SESSION_STATE_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_STATE_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->fLeftSample = inStateInfo.fLeftSample;
	sdkInfo->fRightSample = inStateInfo.fRightSample;
	sdkInfo->fLoTrack = inStateInfo.fLoTrack;
	sdkInfo->fHiTrack = inStateInfo.fHiTrack;
	sdkInfo->qiLoSample = inStateInfo.qiLoSample;
	sdkInfo->qiHiSample = inStateInfo.qiHiSample;
	sdkInfo->fNewLoTrack = inStateInfo.fNewLoTrack;

	return sdkInfo;
}

SDK_SESSION_TEMPO_INFO* SessionIterpretter_ToSDK::TempoInfoForSdk(const SESSION_TEMPO_INFO_NEW& inTempoInfo)
{
	// First, create the sdk object
	SDK_SESSION_TEMPO_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_TEMPO_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->fBeatsPerMinute = inTempoInfo.fBeatsPerMinute;
	sdkInfo->iBeatsPerBar = inTempoInfo.iBeatsPerBar;
	sdkInfo->iTicksPerQuarterNote = inTempoInfo.iTicksPerQuarterNote;
	sdkInfo->fBeatOffsetMs = inTempoInfo.fBeatOffsetMs;
	sdkInfo->iSessionBaseNote = inTempoInfo.iSessionBaseNote;
	sdkInfo->iNoteLen = inTempoInfo.iNoteLen;

	return sdkInfo;
}

SDK_SESSION_ALL_TRACKS_INFO* SessionIterpretter_ToSDK::TracksInfoForSdk(const SESSION_ALL_TRACKS_INFO_NEW& inTracksInfo)
{
	// First, create the sdk object
	SDK_SESSION_ALL_TRACKS_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_ALL_TRACKS_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->nextTrackID = inTracksInfo.nextTrackID;
	sdkInfo->masterTrack = FullAudioTrackInfoForSdk(inTracksInfo.masterTrack, mAllocationList);
	sdkInfo->audioTracks = FullAudioTracksInfoForSdk(inTracksInfo.audioTracks, sdkInfo->numAudioTracks, mAllocationList);
	sdkInfo->nonAudioTracks = FullCommonTracksInfoForSdk(inTracksInfo.nonAudioTracks, sdkInfo->numNonAudioTracks, mAllocationList);

	return sdkInfo;
}

SDK_SESSION_ENTITY_INFO** SessionIterpretter_ToSDK::WaveEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inWaveEntityInfo, unsigned int& outNumEntities)
{
	return EntityInfoForSdk(inWaveEntityInfo, outNumEntities, mAllocationList);
}

SDK_SESSION_ENTITY_INFO** SessionIterpretter_ToSDK::MidiEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inMidiEntityInfo, unsigned int& outNumEntities)
{
	return EntityInfoForSdk(inMidiEntityInfo, outNumEntities, mAllocationList);
}

SDK_SESSION_ENTITY_INFO** SessionIterpretter_ToSDK::VideoEntityInfoForSdk(const std::vector<SESSION_ENTITY_INFO_NEW>& inVideoEntityInfo, unsigned int& outNumEntities)
{
	return EntityInfoForSdk(inVideoEntityInfo, outNumEntities, mAllocationList);
}

SDK_SESSION_GROUP_INFO** SessionIterpretter_ToSDK::GroupInfoForSdk(const std::vector<SESSION_GROUP_INFO_NEW>& inGroupInfo, unsigned int& outNumGroups)
{
	outNumGroups = inGroupInfo.size();

	if (outNumGroups == 0)
	{
		return 0;
	}

	SDK_SESSION_GROUP_INFO** outTracks = CreateSdkList<SDK_SESSION_GROUP_INFO*>(outNumGroups, mAllocationList);

	std::vector<SESSION_GROUP_INFO_NEW>::const_iterator iter = inGroupInfo.begin();
	const std::vector<SESSION_GROUP_INFO_NEW>::const_iterator endIter = inGroupInfo.end();
	for (u32 index = 0; iter != endIter && index < outNumGroups; ++iter, ++index)
	{
		SDK_SESSION_GROUP_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_GROUP_INFO>(mAllocationList);
		sdkInfo->iIndex = iter->iIndex;
		sdkInfo->iColor = iter->iColor;

		outTracks[index] = sdkInfo;
	}
	assert(iter == endIter && index == outNumGroups);

	return outTracks;
}

SDK_SESSION_BLOCK_INFO* SessionIterpretter_ToSDK::BlockInfoForSdk(const SESSION_BLOCK_INFO_NEW& inBlockInfo)
{
	// First, create the sdk object
	SDK_SESSION_BLOCK_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_BLOCK_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->commonBlockInfo = CommonBlockInfoForSdk(inBlockInfo.commonBlockInfo, sdkInfo->numCommonBlockInfo, mAllocationList);
	sdkInfo->waveBlockInfo = WaveBlockInfoForSdk(inBlockInfo.waveBlockInfo, sdkInfo->numWaveBlockInfo, mAllocationList);
	sdkInfo->videoBlockInfo = VideoBlockInfoForSdk(inBlockInfo.videoBlockInfo, sdkInfo->numVideoBlockInfo, mAllocationList);

	return sdkInfo;
}

SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO** SessionIterpretter_ToSDK::EnvelopeInfoForSdk(const std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>& inEnvelopeInfo, unsigned int& outNumEnvelopeGroups)
{
	outNumEnvelopeGroups = inEnvelopeInfo.size();

	if (outNumEnvelopeGroups == 0)
	{
		return 0;
	}

	SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO** outEnvelopes = CreateSdkList<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO*>(outNumEnvelopeGroups, mAllocationList);

	std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>::const_iterator iter = inEnvelopeInfo.begin();
	const std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW>::const_iterator endIter = inEnvelopeInfo.end();
	for (u32 index = 0; iter != endIter && index < outNumEnvelopeGroups; ++iter, ++index)
	{
		SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_BLOCK_ENVELOPE_GROUP_INFO>(mAllocationList);
		sdkInfo->dwID = iter->dwID;
		sdkInfo->rightMostSampleInEnvelopes = iter->rightMostSampleInEnvelope;
		sdkInfo->envelopes = BlockEnvelopesForSdk(iter->envelopes, sdkInfo->numEnvelopes, mAllocationList);

		outEnvelopes[index] = sdkInfo;
	}
	assert(iter == endIter && index == outNumEnvelopeGroups);

	return outEnvelopes;
}

SDK_SESSION_LOOP_INFO** SessionIterpretter_ToSDK::LoopInfoForSdk(const std::vector<SESSION_LOOP_INFO_NEW>& inLoopInfo, unsigned int& outNumLoops)
{
	outNumLoops = inLoopInfo.size();

	if (outNumLoops == 0)
	{
		return 0;
	}

	SDK_SESSION_LOOP_INFO** outLoops = CreateSdkList<SDK_SESSION_LOOP_INFO*>(outNumLoops, mAllocationList);

	std::vector<SESSION_LOOP_INFO_NEW>::const_iterator iter = inLoopInfo.begin();
	const std::vector<SESSION_LOOP_INFO_NEW>::const_iterator endIter = inLoopInfo.end();
	for (u32 index = 0; iter != endIter && index < outNumLoops; ++iter, ++index)
	{
		SDK_SESSION_LOOP_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_LOOP_INFO>(mAllocationList);
		sdkInfo->dwID = iter->dwID;
		sdkInfo->fNumBeats = iter->fNumBeats;
		sdkInfo->fNativeBpm = iter->fNativeBpm;
		sdkInfo->iBaseNote = iter->iBaseNote;
		sdkInfo->bLoopSimple = BoolForSdk(iter->bLoopSimple);
		sdkInfo->bLoopInSamples = BoolForSdk(iter->bLoopInSamples);
		sdkInfo->dwLoopSamples = iter->dwLoopSamples;
		sdkInfo->bLoopInBeats = BoolForSdk(iter->bLoopInBeats);
		sdkInfo->fLoopEveryNBeats = iter->fLoopEveryNBeats;
		sdkInfo->bFollowSessionTempo = BoolForSdk(iter->bFollowSessionTempo);
		sdkInfo->fLoopTempo = iter->fLoopTempo;
		sdkInfo->bTempoMatchFixed = BoolForSdk(iter->bTempoMatchFixed);
		sdkInfo->bTempoMatchStretch = BoolForSdk(iter->bTempoMatchStretch);
		sdkInfo->bTempoMatchResample = BoolForSdk(iter->bTempoMatchResample);
		sdkInfo->bTempoMatchBeatSplice = BoolForSdk(iter->bTempoMatchBeatSplice);
		sdkInfo->iBeatSpliceRate = iter->iBeatSpliceRate;
		sdkInfo->bCrossfade = BoolForSdk(iter->bCrossfade);
		sdkInfo->fSessionTempoUsed = iter->fSessionTempoUsed;
		sdkInfo->iStretchQuality = iter->iStretchQuality;
		sdkInfo->fStretchSplicesPerBeat = iter->fStretchSplicesPerBeat;
		sdkInfo->fStretchOverlapping = iter->fStretchOverlapping;
		sdkInfo->bBeatSpliceAutoFind = BoolForSdk(iter->bBeatSpliceAutoFind);
		sdkInfo->fBeatSpliceDb = iter->fBeatSpliceDb;
		sdkInfo->fBeatSpliceMs = iter->fBeatSpliceMs;
		sdkInfo->dwOrigFileIndent = iter->dwOrigFileIndent;
		sdkInfo->fTranspose = iter->fTranspose;
		sdkInfo->bLoopOnce = BoolForSdk(iter->bLoopOnce);

		outLoops[index] = sdkInfo;
	}
	assert(iter == endIter && index == outNumLoops);

	return outLoops;
}

SDK_SESSION_CUE_INFO** SessionIterpretter_ToSDK::CueInfoForSdk(const std::vector<SESSION_CUE_INFO_NEW>& inCueInfo, unsigned int& outNumCues)
{
	outNumCues = inCueInfo.size();

	if (outNumCues == 0)
	{
		return 0;
	}

	SDK_SESSION_CUE_INFO** outCues = CreateSdkList<SDK_SESSION_CUE_INFO*>(outNumCues, mAllocationList);

	std::vector<SESSION_CUE_INFO_NEW>::const_iterator iter = inCueInfo.begin();
	const std::vector<SESSION_CUE_INFO_NEW>::const_iterator endIter = inCueInfo.end();
	for (u32 index = 0; iter != endIter && index < outNumCues; ++iter, ++index)
	{
		SDK_SESSION_CUE_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_CUE_INFO>(mAllocationList);
		sdkInfo->dwType = iter->dwType;
		sdkInfo->qiOffset = iter->qiOffset;
		sdkInfo->qiLength = iter->qiLength;
		sdkInfo->wszName = StringForSdk(iter->wszName, mAllocationList);
		sdkInfo->wszDesc = StringForSdk(iter->wszDesc, mAllocationList);

		outCues[index] = sdkInfo;
	}
	assert(iter == endIter && index == outNumCues);

	return outCues;
}

SDK_SESSION_METRONOME_INFO* SessionIterpretter_ToSDK::MetronomeInfoForSdk(const SESSION_METRONOME_INFO_NEW& inMetronomeInfo)
{
	// First, create the sdk object
	SDK_SESSION_METRONOME_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_METRONOME_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->iSampleRate = inMetronomeInfo.iSampleRate;
	sdkInfo->fBPM = inMetronomeInfo.fBPM;
	sdkInfo->fOffsetMs = inMetronomeInfo.fOffsetMs;
	sdkInfo->fVolume = inMetronomeInfo.fVolume;
	sdkInfo->iTickSpacing = inMetronomeInfo.iTickSpacing;
	sdkInfo->iOffset = inMetronomeInfo.iOffset;
	sdkInfo->iSet = inMetronomeInfo.iSet;
	sdkInfo->bEnable = BoolForSdk(inMetronomeInfo.bEnable);
	sdkInfo->szDefaultTime = StringForSdk(inMetronomeInfo.szDefaultTime, mAllocationList);
	sdkInfo->szPattern = StringForSdk(inMetronomeInfo.szPattern, mAllocationList);
	sdkInfo->dwOutputPort = inMetronomeInfo.dwOutputPort;
	sdkInfo->iNoteLen = inMetronomeInfo.iNoteLen;

	return sdkInfo;
}

SDK_SESSION_MULTICHANNELENCODER_INFO* SessionIterpretter_ToSDK::MultiChannelEncoderInfoForSdk(const SESSION_MULTICHANNELENCODER_INFO_NEW& inMultiChannelEncoderInfo)
{
	// First, create the sdk object
	SDK_SESSION_MULTICHANNELENCODER_INFO* sdkInfo = CreateSdkObject<SDK_SESSION_MULTICHANNELENCODER_INFO>(mAllocationList);

	// Convert between our common struct and the SDK struct
	sdkInfo->fMasterVolume = inMultiChannelEncoderInfo.fMasterVolume;
	sdkInfo->sessionName = StringForSdk(inMultiChannelEncoderInfo.sessionName, mAllocationList);
	sdkInfo->tracks = MultiChannelEncoderTrackForSdk(inMultiChannelEncoderInfo.tracks, sdkInfo->numTracks, mAllocationList);

	return sdkInfo;
}


}		// namespace AuditionSessionSDK
