/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2007 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

//
// SessionSerializerStructs.h
//

#pragma once


#include <string>
#include <vector>
#include <bitset>
#include <Guiddef.h>


// Some of these settings should match up to the tempo information (SESSION_TEMPO_INFO_NEW)
// Other settings match up to the metronome info in the Advanced Session Properties dialog > Metronome tab.
struct SESSION_METRONOME_INFO_NEW
{
	int				iSampleRate;			// This should match the session sample rate (SESSION_MAIN_INFO_NEW::iSampleRate)
	double			fBPM;					// This should match SESSION_TEMPO_INFO_NEW::fBeatsPerMinutes
	double			fOffsetMs;				// This should match SESSION_TEMPO_INFO_NEW::fBeatOffsetMs
	double			fVolume;				// This matches up with the volume settings in the Advanced Session Properties dialog > Metronome tab.
	int				iTickSpacing;			// The number of samples per tick (usually sampleRate / 2)
	int				iOffset;				// The same measurement as fOffsetMs above, but in samples.  (so if fOffsetMs is 100 (ms) and the sample rate is 44100, this should be 4410 (samples))
	int				iSet;					// 0-based index of the sound set dropdown list in the Advanced Session Properties dialog > Metronome tab.
	bool			bEnable;				// Is the metronome turned on?
	std::wstring	szDefaultTime;			// The name of the metronome time signature (in the Advanced Session Properties dialog > Metronome tab.)
	std::wstring	szPattern;				// The pattern used for the metronome (matches with the Pattern field in the Advanced Session Properties dialog > Metronome tab.)
	unsigned long	dwOutputPort;			// Output port to use on the current audio device
	int				iNoteLen;				// This should match SESSION_TEMPO_INFO_NEW::iNoteLen
};


struct SESSION_ENVELOPE_POINT_INFO_NEW
{
	__int64 dwAt;				// sample position of this point
	double value;				// normalized value (from 0.0 to 1.0 - from the UI perspective, 0.0 would show the point at the bottom of the block, 1.0 would show the point at the top of the block)
};


struct SESSION_BLOCK_ENVELOPE_INFO_NEW
{
	unsigned int uiType;		// (ET_PAN, ET_VOL, etc.  See below)
	bool bSplined;				// whether or not the envelope is splined
	std::vector<SESSION_ENVELOPE_POINT_INFO_NEW> points;
};

// Envelope types  (use these values for SESSION_BLOCK_ENVELOPE_INFO_NEW::uiType)
#define F_ET_VOL		0x01		// Volume			(or Front/Back for surround encoder envelopes - (1.0 = front, 0.0 = back))
#define F_ET_PAN		0x02		// Pan				(or Left/Right for surround encoder envelopes - (1.0 = left, 0.0 = right))
#define F_ET_DRY		0x04		// Wet/Dry (obsolete - Audition will create this envelope type for backwards compatibility, but it can be ignored)


// Every block is expected to have an envelope group.
// For video blocks, 'envelopes' should be empty
// For audio blocks, 'envelopes' should contain 1 envelope for volume and 1 envelope for pan.
struct SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW
{
	// For block envelopes - Identifies the block(clip) that these envelopes are for - id matches up with SESSION_COMMON_BLOCK_INFO_NEW::dwBlockID
	// For track envelops in the surround encoder (multichannel encoder), should simply match up to SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW::iTrack
	unsigned long dwID;

	std::vector<SESSION_BLOCK_ENVELOPE_INFO_NEW> envelopes;		// envelope info
	__int64 rightMostSampleInEnvelope;		// This should be set to the largest envelops::points::dwAt value.  (pretty much the same as the length of the longest envelope)
};


struct SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW
{
	bool bUseFlatLines;				// inversely matches "Pan Envelopes" setting in the Surround Encoder dialog (set to false if "Pan Envelopes" should be checked)
	double fFlatFade;				// if bUseFlatLines is true, this is used to determine front / back relationship (1.0 = front, 0.0 = back)
	double fFlatPan;				// if bUseFlatLines is true, this is used to determine left / right panning (1.0 = left, 0.0 = right)
	int iSubLevel;					// sub level (0 to 100)
	int iCenterLevel;				// center level (0 to 100)
	int iTrackVolume;				// overall track volume (0 to 100)
	unsigned long dwActiveSpeakers;		// bit field of active speakers	(see speakers below)
	int iPanningMode;				// special panning mode (0 for stereo, 1 for summed-to-mono)
	bool bFixedRouting;				// if true, all output goes to active speakers (otherwise use panning envelopes)
	int iTrack;						// 0-based index of track, as visible in the multitrack ui (master track is always the last track, so it has the highest index number).   (slightly different from SESSION_AUDIO_TRACK_FULL_INFO_NEW::trackOrderIndex, as it uses a 1-based index and the master is always 0)
	SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW envelopeGroup;	// envelopes for the track - each track should have two envelops in this group, one for front/back panning, one for left/right panning.
};

// speaker flags for SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW::dwActiveSpeakers
#define SPEAKER_FRONT_LEFT              0x1
#define SPEAKER_FRONT_RIGHT             0x2
#define SPEAKER_FRONT_CENTER            0x4
#define SPEAKER_LOW_FREQUENCY           0x8
#define SPEAKER_BACK_LEFT               0x10
#define SPEAKER_BACK_RIGHT              0x20

struct SESSION_MULTICHANNELENCODER_INFO_NEW
{
	std::vector<SESSION_MULTICHANNELENCODER_TRACK_INFO_NEW>	tracks;
	std::wstring sessionName;		// unused - just leave blank
	double fMasterVolume;			// Matches with "Master Level" in the Surround Encoder dialog.  (for 0dB, set to 100.0, for +6dB, set to 200.0, for -infdB, set to 0.0)
};


// Most of the following corresponds to settings found in the Loop Properties dialog (Clip menu > Loop Properties)
struct SESSION_LOOP_INFO_NEW
{
	unsigned long dwID;			// Block ID for the block (clip) that is associated with this loop information  (SESSION_COMMON_BLOCK_INFO_NEW::dwBlockID)
	double fNumBeats;			// Corresponds to the "Number of Beats" field in the Loop Properties dialog
	double fNativeBpm;			// Corresponds to the "Tempo" field in the Loop Properties dialog
	int iBaseNote;				// Corresponds to the "Key" field in the Loop Properties dialog.  Use the kSessionBaseNote enum below (with the tempo info) for this
	bool bLoopSimple;			// Corresponds to the "Simple Looping" option in the Loop Properties dialog (both bLoopInSamples and bLoopInBeats should both be false)
	bool bLoopInSamples;		// Corresponds to the "Repeat every N seconds" option in the Loop Properties dialog (both bLoopSimple and bLoopInBeats should be false)
	unsigned long dwLoopSamples;	// How often to repeat (in samples) for the "Repeat every N seconds" option in the Loop Properties dialog
	bool bLoopInBeats;			// Corresponds to the "Repeat every N beats" option in the Loop Properties dialog (both bLoopInSamples and bLoopSimple should be false)
	double fLoopEveryNBeats;	// Corresponds to the "seconds" field for the "Repeat every N beats" option in the Loop Properties dialog
	bool bFollowSessionTempo;	// Corresponds to the "Follow Session Tempo" field in the Loop Properties dialog
	double fLoopTempo;			// Corresponds to the "beats" field for the "Repeat every N beats" option in the Loop Properties dialog
	bool bTempoMatchFixed;		// Corresponds to the "Fixed Length" setting for the "Tempo Matching" option in the Loop Properties dialog  (if set to true, bTempoMatchStretch, bTempoMatchResample, and bTempoMatchBeatSplice should be false)
	bool bTempoMatchStretch;	// Corresponds to the "Time-Scale Stretch" setting for the "Tempo Matching" option in the Loop Properties dialog (if set to true, bTempoMatchFixed, bTempoMatchResample, and bTempoMatchBeatSplice should be false) (*see note below about "Hybrid" option)
	bool bTempoMatchResample;	// Corresponds to the "Resample" setting for the "Tempo Matching" option in the Loop Properties dialog  (if set to true, bTempoMatchFixed, bTempoMatchStretch, and bTempoMatchBeatSplice should be false)
	bool bTempoMatchBeatSplice;	// Corresponds to the "Beat Splice" setting for the "Tempo Matching" option in the Loop Properties dialog  (if set to true, bTempoMatchFixed, bTempoMatchStretch, and bTempoMatchResample should be false) (*see note below about "Hybrid" option)
								// if bTempoMatchBeatSplice and bTempoMatchStretch are both true, then Audition uses the "Hybrid" setting for Tempo Matching
	int iBeatSpliceRate;		// if bTempoMatchFixed or bTempoMatchStretch, set to 4.  if bTempoMatchResample, set to 1. if bTempoMatchBeatSplice, set to 2. if both bTempoMatchStretch and bTempoMatchBeatSplice (hybrid), set to 4.
	bool bCrossfade;			// obsolete.  Set to false.
	double fSessionTempoUsed;	// This should be set to SESSION_TEMPO_INFO_NEW::fBeatsPerMinute
	int iStretchQuality;		// Corresponds to the "Quality" setting for the "Tempo Matching" option (available when "Time-Scale Stretch" or "Resample" is selected)
	double fStretchSplicesPerBeat;	// Corresponds to the "Solo Voice / Instrument" field for the "Time-Scale Stretch" setting for the "Tempo Matching" option in the Loop Properties dialog.  (if non-zero, then true (checked), if zero, then false (unchecked)
	double fStretchOverlapping;	// Corresponds to the "Preserve Formant" field for the "Time-Scale Stretch" setting for the "Tempo Matching" option in the Loop Properties dialog.  (if non-zero, then true (checked), if zero, then false (unchecked)
	bool bBeatSpliceAutoFind;	// Corresponds to the "Auto-find beats" option of the "Beat Splice" setting for the "Tempo Matching" option in the Loop Properties dialog.  If true, "Auto-find beats" is selected.  If false, "Use file's beat markers" is selected.
	double fBeatSpliceDb;		// Corresponds to the "dB" field for the "Auto-find beats" option of the "Beat Splice" setting for the "Tempo Matching" option in the Loop Properties dialog
	double fBeatSpliceMs;		// Corresponds to the "ms" field for the "Auto-find beats" option of the "Beat Splice" setting for the "Tempo Matching" option in the Loop Properties dialog
	unsigned long dwOrigFileIndent;		// If bLoopOnce is false, then unused: set to 0.  If bLoopOnce is true, then this setting corresponds to the clip indent (in samples) of the original, unstretched source.  (SESSION_COMMON_BLOCK_INFO_NEW::dwFileIndent will contain the file indent, relative to the new stretched audio)
	double fTranspose;			// Corresponds to the "Transpose" field in the Loop Properties dialog.
	bool bLoopOnce;				// If set, instead of being a looped-clip, the clip will be a time-stretch clip.  Most other settings still apply.  dwLoopSamples is used to determine the time stretch.  (if clip was originally 5000 samples, and stretched 110%, then dwLoopSamples would be set to 5500)
};


struct SESSION_GROUP_INFO_NEW
{
	int iIndex;			// group index id - valid indexes are from 1 to 63.	(matches with SESSION_COMMON_BLOCK_INFO_NEW::iParentGroup)
	int iColor;			// the color value found in the Group Color dialog (Clip menu > Group Color).  default is 186.
};


struct VERSION_INFO_NEW
{
	double version;					// Audition version (e.g. 3.0)
	double buildNumber;				// Audition build number (use 0.0, as this is currently not used)
	std::wstring szVersionStringInfo;		// Audition version string (e.g. "Audition 3.0 [0.0]"
};


struct SESSION_ENV_INFO_NEW
{
	unsigned int uiType;					// typically set to 0.  Set to 1 for volume faders, and 2 for panning.
	bool bSplined;							// unused. set to false
	std::wstring	szName;					// name of the parameter that owns this envelope
	unsigned long	dwFlags;				// unused.  set to 0
	unsigned long	LineColor;				// an RGB value in the COLORREF format
	__int64	dwSize;							// How many samples long is the envelope
	bool	bReadOnly;						// This should match the SESSION_AUDIO_PARAM_NEW::bSafeDuringWrite value.  (i.e. if safe during write, then also read only)
	std::vector<SESSION_ENVELOPE_POINT_INFO_NEW> points;
};


struct SESSION_AUDIO_PARAM_NEW
{
	bool	bIsAutomatable;							// true if the parameter is automatable, false if not
	double	parameterValueForRendering;				// the current value (should match up to the value in the envelope at sample position SESSION_STATE_INFO_NEW::qiLowSample)
	bool	bSafeDuringWrite;						// true if the option for "safe during write" is set for this parameter.  otherwise, false.
	SESSION_ENV_INFO_NEW	envelope;				// the parameter automation envelope
};


struct SESSION_AUDIO_COMPONENT_NEW
{
	unsigned int componentType;						// see kComponentType enum, below
	__int64 initialDelay;							// set to 0
	bool isBypass;									// true if turned off, bypassed, disabled.  false if this component is on / enabled.
	bool isSerial;									// set to true
	std::vector<SESSION_AUDIO_PARAM_NEW> audioParams;	// automation data for this component's parameters
};


enum kComponentType
{
	kAudioComponentType_Input		= 0,		// input gain
	kAudioComponentType_Fader		= 1,		// volume
	kAudioComponentType_Meter		= 2,		// meter
	kAudioComponentType_Send		= 3,		// send
	kAudioComponentType_Effect		= 4,		// insert effects (requires a guidString to identify the effect)
	kAudioComponentType_EQ			= 5,		// track eq
	kAudioComponentType_Mute		= 6,		// mute
	kAudioComponentType_Output		= 7,		// pan
	kAudioComponentType_EffectGroup	= 8			// insert effects group
};


struct SESSION_AUDIO_OUTPUT_COMPONENT_NEW
{
	unsigned int componentType;						// set to kAudioComponentType_Output
	__int64 initialDelay;							// set to 0
	bool isBypass;									// set to false
	std::vector<SESSION_AUDIO_PARAM_NEW> audioParams;	// automation data for this component's parameters
	unsigned int pannerMode;						// should always be set to 1 if the track output device is stereo, 0 if the track output device is mono
	bool isSumDown;									// true if the track output is summed to mono.  false if not summed to mono.
	unsigned int outputChannels;					// should either be set to 2 (if outputting to a stereo device) or 1 (if outputting to a mono device)
	int busID;										// set to 0 (if this track outputs to a bus, then the busID is set in the SESSION_AUDIO_TRACK_INFO_NEW::outputPort field)
};


// The following component is used for VST effects applied to a track
struct SESSION_AUDIO_EFFECT_COMPONENT_NEW
{
	unsigned int componentType;						// set to kAudioComponentType_Effect
	__int64 initialDelay;							// set to 0
	bool isBypass;									// true if turned off, bypassed, disabled.  false if this component is on / enabled.
	std::vector<SESSION_AUDIO_PARAM_NEW> audioParams;	// automation data for this component's parameters
	GUID guid;										// The GUID of the vst effect
	std::wstring name;								// used for reading in older .ses sessions  (also used as a backup if no match is found using the GUID)
	std::wstring groupName;							// used for reading in older .ses sessions  (also used as a backup if no match is found using the GUID)
	std::wstring effectParamsString;				// A string representation of the binary data chunk for the effect parameters
};


struct SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW
{
	unsigned int componentType;							// set to kAudioComponentType_EffectGroup
	__int64 initialDelay;								// set to 0
	bool isBypass;										// true if turned off, bypassed, disabled.  false if this component is on / enabled.
	bool isSerial;										// set to true
	std::vector<SESSION_AUDIO_PARAM_NEW> audioParams;	// automation data for this component's parameters
	SESSION_AUDIO_COMPONENT_NEW inputGain;				// uses a component type of 0
	SESSION_AUDIO_COMPONENT_NEW outputGain;				// uses a component type of 0
};


struct SESSION_AUDIO_SEND_COMPONENT_NEW
{
	unsigned int componentType;							// set to kAudioComponentType_Send
	__int64 initialDelay;								// set to 0
	bool isBypass;										// true if turned off, bypassed, disabled.  false if this component is on / enabled.
	std::vector<SESSION_AUDIO_PARAM_NEW> audioParams;	// automation data for this component's parameters
	unsigned int pannerMode;							// set to 1
	bool isSumDown;										// set to false
	unsigned int numOutputChannels;						// set to 2
	int busID;											// set to 0 if this send is unused.  If this send is used, this should match up with SESSION_AUDIO_TRACK_INFO_NEW::busID for the buss that is used
	bool isPre;											// true if the send is pre-fader, false if the send is post-fader
};


struct SESSION_COMMON_TRACK_INFO_NEW
{
	// valid ids for audio tracks should start from 10000 (up to 239999)
	// valid id for the video track (there can be only one) is 490000
	unsigned long	id;

	unsigned long	dwFlags;			// See track flags below

	std::wstring	szTitle;			// track title
	unsigned long	dwTrackHeight;		// track height (in pixels)
};


// Track flags (for SESSION_COMMON_TRACK_INFO_NEW::dwFlags)
#define F_TRACK_MUTE		0x01
#define F_TRACK_SOLO		0x02
#define F_TRACK_RECORD		0x04
#define F_TRACK_FROZEN		0x08
#define F_PRE_METER			0x10
#define F_PRE_EFFECT		0x20
#define F_TRACK_EQ_UI		0x40
#define F_EFFECTS_RACK_UI	0x80


struct SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW
{
	unsigned int	audioComponentType;			// see kComponentType enum, above
	std::wstring	guidString;					// if audioComponentType is 4, then this should be the effect's guid (e.g. "{EA93BBBE-0B8F-47D6-AC6D-B67B45687047}").  If the audioComponentType is not 4, then this should be set to "no guid"
	std::wstring	componentName;				// only really needed with kComponentType_Send.  ("Send 1 Output:", "Send 2 Output:", etc...)
	unsigned int	parameterIndex;				// the parameter of the component
	unsigned int	laneUIHeight;				// laneUIHeight (in pixels)
};


struct SESSION_AUDIO_TRACK_INFO_NEW
{
	SESSION_COMMON_TRACK_INFO_NEW baseTrackInfo;

	// trackType:
	// TRACK_CONTENT = 0		// standard audio track with clips and such
	// TRACK_BUS = 1			// bus track
	// TRACK_MASTER = 2			// master track
	// TRACK_MIDIHOST = 3		// midi host track
	unsigned int	trackType;

	// The following identifies the bus ID, if the track is a bus
	// If the track is not a bus, set this to 0
	// If the track is the master track (technically a bus), set this to 20000
	// If the track is a bus track, set this to a bus ID, starting at 20001
	unsigned int	busID;

	// automationMode:
	// AUTOMATION_OFF = 0
	// AUTOMATION_READ = 1
	// AUTOMATION_WRITE = 2
	// AUTOMATION_LATCH = 3
	// AUTOMATION_TOUCH = 4
	unsigned int	automationMode;

	std::wstring	inputDriver;			// The selected sound card.  (should match up with SESSION_MAIN_INFO_NEW::wszASIODriver)
	unsigned int	inputPort;				// The input port number used
	std::wstring	inputPortName;			// The name of the input port
	std::wstring	outputDriver;			// The selected sound card.  (should match up with SESSION_MAIN_INFO_NEW::wszASIODriver)
	unsigned int	outputPort;				// The busID of the target bus if outputting to a bus.  The output port number if outputting to an audio device
	std::wstring	outputPortName;			// The name of the output port
	bool			isPreMeter;				// set to true if the track is armed for record (arming a track for record is done by setting a flag on SESSION_COMMON_TRACK_INFO_NEW::dwFlags)
	bool			isPreEffect;			// determines whether the effects are pre-fader() or post-fader()
	bool			isFreeze;				// is the track frozen?
	bool			isTrackEQUI;			// not used - set to false
	bool			isEffectsRackUI;		// not used - set to false
	SESSION_AUDIO_COMPONENT_NEW					inputComponent;		// componentType should be set to kAudioComponentType_Input
	SESSION_AUDIO_COMPONENT_NEW					eqComponent;		// componentType should be set to kAudioComponentType_EQ
	SESSION_AUDIO_COMPONENT_NEW					faderComponent;		// componentType should be set to kAudioComponentType_Fader
	SESSION_AUDIO_COMPONENT_NEW					muteComponent;		// componentType should be set to kAudioComponentType_Mute
	SESSION_AUDIO_OUTPUT_COMPONENT_NEW			outputComponent;	// see SESSION_AUDIO_OUTPUT_COMPONENT_NEW, above
	SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW	effectComponentGroup;	// see SESSION_AUDIO_EFFECT_COMPONENT_GROUP_NEW, above
	std::vector<SESSION_AUDIO_SEND_COMPONENT_NEW>	sendComponents;		// see SESSION_AUDIO_SEND_COMPONENT_NEW, above
	std::vector<SESSION_AUDIO_EFFECT_COMPONENT_NEW> effectComponents;	// see SESSION_AUDIO_EFFECT_COMPONENT_NEW, above
	std::vector<bool>	hasEffectInSlot;		// This determines what slots the effects (in effectComponents) use.  This is expected to be a vector of 16 elements, all should be false except for where an effect exists.  So if only the 3rd item in this vector is true, then this implies a single effect and it's placement is in the 3rd slot.
	bool			isShowingAutomationLanes;	// not used - set to false
	unsigned int	trackHeight;				// track height (in pixels)
	std::vector<SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW>	automationLanes;	// see SESSION_AUDIO_TRACK_AUTOMATION_LANE_INFO_NEW, above

	// The following is ignored unless trackType == TRACK_MIDIHOST
	SESSION_AUDIO_EFFECT_COMPONENT_NEW			midiHostEffect;			// see SESSION_AUDIO_EFFECT_COMPONENT_NEW, above
};


struct SESSION_AUDIO_TRACK_FULL_INFO_NEW
{
	// trackType:
	// Track_Audio = 0				// audio track
	// Track_Bus = 1				// audio track
	// Track_Master = 2				// audio track
	// Track_OBSOLETE = 3			// obsolete - do not use
	// Track_Video = 4				// non-audio track (only has common track info)
	// Track_MIDIHost = 5			// audio track
	unsigned int	trackType;

	// trackOrderIndex determines how the tracks are ordered in the UI.
	// The master track is always 0.  Other tracks start their ordering at 1.
	unsigned int	trackOrderIndex;

	// the rest of the track information
	SESSION_AUDIO_TRACK_INFO_NEW track;
};


struct SESSION_COMMON_TRACK_FULL_INFO_NEW
{
	unsigned int	trackType;				// same as SESSION_AUDIO_TRACK_FULL_INFO_NEW, above (should always be Track_Video, since this should always represent a video track)
	unsigned int	trackOrderIndex;		// same as SESSION_AUDIO_TRACK_FULL_INFO_NEW, above (should always be 1, since this should always represent a video track)
	SESSION_COMMON_TRACK_INFO_NEW track;	// the rest of the track information
};


struct SESSION_ALL_TRACKS_INFO_NEW
{
	// every track has an id.  If a session has 5 tracks, the ids might be 0, 1, 2, 3, and 4, and
	// nextTrackID would be 5.  In this case, nextTrackID is equal to the number of tracks.
	// However, if a user deletes tracks 2 and 3, then the track ids would be 0, 1, and 4, and
	// nextTrackID would still be 5.  To keep things simple, track ids are not reused.
	// So nextTrackID should be equal to the largest track id, plus 1.
	unsigned int	nextTrackID;

	SESSION_AUDIO_TRACK_FULL_INFO_NEW masterTrack;						// master track
	std::vector<SESSION_AUDIO_TRACK_FULL_INFO_NEW> audioTracks;			// audio tracks
	std::vector<SESSION_COMMON_TRACK_FULL_INFO_NEW> nonAudioTracks;		// video tracks (there should be either 0 or 1)
};


// Note:  Cues are now referred to as "Markers"
struct SESSION_CUE_INFO_NEW
{
	unsigned long dwType;		// see type defines, below
	__int64 qiOffset;			// position (in samples)
	__int64 qiLength;			// length (in samples)
	std::wstring wszName;		// cue/marker name
	std::wstring wszDesc;		// cue/marker description
};

#define CUECCTYPE(a, b, c, d) (((DWORD)d<<24) | ((DWORD)c<<16) | ((DWORD)b<<8) | ((DWORD)a))
#define CUECC_RGN		CUECCTYPE('r', 'g', 'n', ' ')		// Standard cue/marker
#define CUECC_TRAK	CUECCTYPE('t', 'r', 'a', 'k')		// CD track marker
#define CUECC_INDX	CUECCTYPE('i', 'n', 'd', 'x')		// CD index marker


struct SESSION_COMMON_BLOCK_INFO_NEW
{
	unsigned long dwBlockID;	// block id - this matches up with SESSION_WAVE_BLOCK_INFO_NEW::dwBlockID or SESSION_VIDEO_BLOCK_INFO_NEW::dwBlockID
	unsigned long dwEntityID;	// entity id - this matches up with SESSION_ENTITY_INFO_NEW::dwWaveformID
	double fLeftVol;			// volume multiplier (for normal volume, set to 1.  for +6 dB, set to 2.  for -6 dB, set to 0.5.  etc...)  fLeftVol and fRightVol should be identical.
	double fRightVol;			// volume multiplier (for normal volume, set to 1.  for +6 dB, set to 2.  for -6 dB, set to 0.5.  etc...)  fLeftVol and fRightVol should be identical.
	__int64 dwOffset;			// where the block starts (number of samples from beginning of session)
	__int64 dwSize;				// the size of the block (in samples)
	__int64 dwFileIndent;		// where in the entity does this block start (offset in samples)
	unsigned long dwFlags;		// see block flags, below
	int iTrack;					// track id - this matches up with SESSION_COMMON_TRACK_INFO_NEW::id
	int iParentGroup;			// group id - this can be from 1 to 63 if using a group.  Set to 0 if not using a group.
	int iPunchGeneration;		// set to 0
	int iPreviousPunch;			// set to 0
	int iNextPunch;				// set to 0
	int iHue;					// the color value found in the Clip Color dialog (Clip menu > Clip Color).  audio block default is 102.  video block default is 140.
	int	iParentBlock;			// set to 0
};



// Block flags (used in SESSION_COMMON_BLOCK_INFO_NEW::dwFlags)

#define F_BLOCK_OPAQUE			0x0001	// If set, any blocks hidden by this block overlapping are not played
#define F_BLOCK_LOCKED			0x0002
#define F_BLOCK_MUTE			0x0004
#define F_BLOCK_NORECORD		0x0008	// if track marked for record, a NORECORD will NOT be recorded into
#define F_BLOCK_RECORD			0x0010 	// if track marked for playback, a RECORD will record into waveblock anyways
#define F_BLOCK_PUNCHIN			0x0020	// if recording into this track, we must first save current, and create new blank
#define F_BLOCK_TOUCHED			0x0040	// with operations that loop through all waves and don't want to touch a wave block twice (multdispcutwaveform)
#define F_BLOCK_UNUSED			0x0080	// if still unused after recording, delete
#define F_BLOCK_JUSTRECORDED	0x0100	// after recording, if this flag set, wave block is merged with one before it if possible
#define F_BLOCK_READINADJUST	0x0200	// when reading in new wave blocks, used internally by ChkReadBlocks
#define F_BLOCK_GROWABLE		0x0400	// if set, block grows automatically while being recorded in to
#define F_BLOCK_HIDDEN			0x0800	// set when a block is part of a punch-in list but is not the currently viewed block
#define F_BLOCK_TEMPOLOCK		0x1000	// If set, and tempo changes, block moves to match
#define F_BLOCK_TEMPOLENLOCK	0x2000	// If set, and tempo changes, block length changes to match



struct SESSION_WAVE_BLOCK_INFO_NEW
{
	unsigned long dwBlockID;		// id - this matches up with SESSION_COMMON_BLOCK_INFO_NEW::dwBlockID
	__int64 iFadeInOffset;			// length (in samples) of the on-clip fade in.  (set to 0 for no fade in)
	__int64 iFadeOutOffset;			// length (in samples) of the on-clip fade out.  (set to 0 for no fade out)
	int iFadeInShapeVal;			// shape of fade in (range:  -100 to 100)
	int iFadeOutShapeVal;			// shape of fade out (range:  -100 to 100)
	int iFadeInType;				// set to 0 for a linear/log fade, 1 for a cosine fade
	int iFadeOutType;				// set to 0 for a linear/log fade, 1 for a cosine fade
	bool hasCrossfadeLeft;			// true if has a crossfade with a different block on the left side
	bool hasCrossfadeRight;			// true if has a crossfade with a different block on the right side
	int iCrossfadeLeftLinkType;		// 0 = not linked, 1 = linked constant gain, 2 = linked symmetrical
	int iCrossfadeRightLinkType;	// 0 = not linked, 1 = linked constant gain, 2 = linked symmetrical
};

struct SESSION_VIDEO_BLOCK_INFO_NEW
{
	unsigned long dwBlockID;			// id - this matches up with SESSION_COMMON_BLOCK_INFO_NEW::dwBlockID
	unsigned short dwFlags;				// set to 1 if wszAudioFileName specifies an audio file that was extracted from this video
	std::wstring wszAudioFileName;		// this should be the filename of the audio file that was extracted from the video file.  If the audio was extracted from the video, but not specifically saved, simply append "Audio for " to the beginning of the video filename for use here.  (video file is "MyVideo.avi", so this would be set to "Audio for MyVideo.avi")
};


// The block information is for audio (wave) blocks and video blocks
// Each block will have specific block info (wave or video) data, and also
// have common block info.
// The specific data structures link to the common block structure by the
// dwBlockID data items in each structure.
// So if there are 5 audio block data structures and 1 video block data structure,
// then there will be 6 common block structures.

struct SESSION_BLOCK_INFO_NEW
{
	std::vector<SESSION_COMMON_BLOCK_INFO_NEW> commonBlockInfo;
	std::vector<SESSION_WAVE_BLOCK_INFO_NEW> waveBlockInfo;
	std::vector<SESSION_VIDEO_BLOCK_INFO_NEW> videoBlockInfo;
};

struct SESSION_ENTITY_INFO_NEW
{
	unsigned long dwWaveformID;			// unique identifier - used with SESSION_COMMON_BLOCK_INFO_NEW::dwEntityId to identify which entities are used for each block
	unsigned long dwFileFormat;			// specifies a specific file format, but you should simply set this to -1
	int iCodecDelay;					// obsolete - set to -1
	__int64 dwTrueLength;				// if an audio entity, set this to the length of the entity, in samples.  If video, set to -1
	unsigned int iEntityType;			// set to 1 if audio file, 3 if video file.
	std::wstring wszFilename;			// filename - can be either: 1) full path, or 2) just filename, if in same folder as session file.
};



struct SESSION_MAIN_INFO_NEW
{
	int				iSampleRate;			// e.g. 44100
	__int64			qiNumSamples;			// session length - number of samples in the session
	double			fMasterVolume;			// mostly obsolete - set to 1.0
	__int64			qiMainOffset;			// see the "SMPTE Start Time Offset" option in the General tab of the Advanced Session Properties dialog. 
	int				iRulerFormat;			// use the kRulerFormat enum below
	int				bPanningMode;			// set to 0 for "L/R Cut Logarithmic", 1 for "-3dB Center, Constant Power"
	double			fVolEnvRange;			// For volume envelopes (always in 0 to 1.0 range), multiply by this factor for true volume
	std::wstring	wszMaster;				// obsolete - leave set as "Master"
	std::wstring	wszASIODriver;			// name of the audio driver selected in the Multitrack View tab of the Audio Hardware Setup dialog. 
	int				iTakeNumber;			// set to 0
	int				iNumberOfBlocks;		// number of audio and video blocks (doesn't count rewire or midi tracks)

	// The following are mainly used for internal purposes only.  For standard use,
	// bRecovery should be set to false and wszFilename can be empty.  If bRecovery is
	// true, that indicates that this session was saved via "auto-save for recovery",
	// and then wszFilename will be used as the sessions filename, rather than the actual
	// name of the file.
	bool			bRecovery;
	std::wstring	wszFilename;
};

enum kRulerFormat
{
	kRulerFormat_HoursMinutesSeconds = 0,
	kRulerFormat_Samples = 1,
	kRulerFormat_Frames = 2,
	kRulerFormat_SMPTE2997Drop = 3,
	kRulerFormat_BarsAndBeats = 4,
	kRulerFormat_SMPTE2997 = 5
};

struct SESSION_STATE_INFO_NEW
{
	// Horizontal view
	double fLeftSample;
	double fRightSample;

	// Vertical view
	double fLoTrack;
	double fHiTrack;
	double fNewLoTrack;

	// Selection
	__int64 qiLoSample;
	__int64 qiHiSample;
};


// The SESSION_TEMPO_INFO_NEW data structure matches options found in the Tempo tab of 
// the Advanced Session Properties dialog, or the Session Properties dockable tab.
//
struct SESSION_TEMPO_INFO_NEW
{
	double fBeatsPerMinute;			// bpm
	int iBeatsPerBar;				// top number in time signature
	int iTicksPerQuarterNote;		// ticks per beat
	double fBeatOffsetMs;			// see the "Song Start" option in the Tempo tab of the Advanced Session Properties dialog. 
	int iSessionBaseNote;			// use the kSessionBaseNote enum below for this
	int iNoteLen;					// bottom number in time signature
};

enum kSessionBaseNote
{
	kSessionBaseNote_None = 0,
	kSessionBaseNote_C		= 60,
	kSessionBaseNote_CSharp	= 61,
	kSessionBaseNote_D		= 62,
	kSessionBaseNote_DSharp = 63,
	kSessionBaseNote_E		= 64,
	kSessionBaseNote_F		= 65,
	kSessionBaseNote_FSharp = 66,
	kSessionBaseNote_G		= 67,
	kSessionBaseNote_GSharp = 68,
	kSessionBaseNote_A		= 69,
	kSessionBaseNote_ASharp = 70,
	kSessionBaseNote_B		= 71,
};

struct COMPLETE_SESSION_INFO_NEW
{
	COMPLETE_SESSION_INFO_NEW()
	{
		sessionMainInfo.first = false;
		versionInfo.first = false;
		notesInfo.first = false;
		stateInfo.first = false;
		tempoInfo.first = false;
		tracksInfo.first = false;
		waveEntityInfo.first = false;
		videoEntityInfo.first = false;
		groupInfo.first = false;
		blockInfo.first = false;
		blockEnvelopes.first = false;
		loopInfo.first = false;
		cueInfo.first = false;
		metronomeInfo.first = false;
		multiChannelEncoderInfo.first = false;
	}

	std::pair<bool, SESSION_MAIN_INFO_NEW>									sessionMainInfo;
	std::pair<bool, VERSION_INFO_NEW>										versionInfo;
	std::pair<bool, std::wstring>											notesInfo;		// See the Notes tab of the Advanced Session Properties dialog
	std::pair<bool, SESSION_STATE_INFO_NEW>									stateInfo;
	std::pair<bool, SESSION_TEMPO_INFO_NEW>									tempoInfo;
	std::pair<bool, SESSION_ALL_TRACKS_INFO_NEW>							tracksInfo;
	std::pair<bool, std::vector<SESSION_ENTITY_INFO_NEW> >					waveEntityInfo;
	std::pair<bool, std::vector<SESSION_ENTITY_INFO_NEW> >					videoEntityInfo;
	std::pair<bool, std::vector<SESSION_GROUP_INFO_NEW> >					groupInfo;
	std::pair<bool, SESSION_BLOCK_INFO_NEW>									blockInfo;
	std::pair<bool, std::vector<SESSION_BLOCK_ENVELOPE_GROUP_INFO_NEW> >	blockEnvelopes;
	std::pair<bool, std::vector<SESSION_LOOP_INFO_NEW> >					loopInfo;
	std::pair<bool, std::vector<SESSION_CUE_INFO_NEW> >						cueInfo;
	std::pair<bool, SESSION_METRONOME_INFO_NEW>								metronomeInfo;
	std::pair<bool, SESSION_MULTICHANNELENCODER_INFO_NEW>					multiChannelEncoderInfo;
};