/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2007 Adobe Systems Incorporated                       */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#include "SimpleParser.h"

#include <stdlib.h>			// for _wtoi64 and _i64tow
#include <Windows.h>
#include <Shlwapi.h>
#include <assert.h>


namespace {

	const std::string kXmlBeginTag("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
	const std::wstring kXmlBeginTagWide(L"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");

	bool GetNextValue(const std::wstring& inString, std::wstring::size_type& ioFirstAvailableCharIndex, std::wstring& outValue)
	{
		bool success = true;

		std::wstring::size_type tagStartIndex = inString.find_first_of(L'<', ioFirstAvailableCharIndex);
		std::wstring::size_type valueStartIndex = inString.find_first_not_of(L"< \n\r\t", ioFirstAvailableCharIndex);
		std::wstring::size_type valueEndIndex = std::wstring::npos;
		std::wstring::size_type trimmedValueEndIndex = std::wstring::npos;
		if (valueStartIndex != std::wstring::npos)
		{
			valueEndIndex = inString.find_first_of(L"<", valueStartIndex);

			// Trim trailing spaces, tabs, etc...
			if (valueEndIndex != std::wstring::npos && valueStartIndex < valueEndIndex)
			{
				trimmedValueEndIndex = inString.find_last_not_of(L"< \n\r\t", valueEndIndex);
			}
		}

		if (valueStartIndex == std::wstring::npos || valueEndIndex == std::wstring::npos || 
			trimmedValueEndIndex == std::wstring::npos || trimmedValueEndIndex < valueStartIndex ||
			(tagStartIndex != std::wstring::npos && tagStartIndex < valueStartIndex))
		{
			success = false;
		}
		else
		{
			// We have a valid value!
			outValue = inString.substr(valueStartIndex, (trimmedValueEndIndex + 1) - valueStartIndex);
			ioFirstAvailableCharIndex = valueEndIndex;
		}

		return success;
	}

	bool GetNextTag(const std::wstring& inString, std::wstring::size_type& ioFirstAvailableCharIndex, std::wstring& outTag)
	{
		bool success = true;

		std::wstring::size_type tagStartIndex = inString.find_first_of(L'<', ioFirstAvailableCharIndex);
		std::wstring::size_type tagEndIndex = std::wstring::npos;
		if (tagStartIndex != std::wstring::npos)
		{
			tagEndIndex = inString.find_first_of(L'>', tagStartIndex);
		}

		if (tagStartIndex == std::wstring::npos || tagEndIndex == std::wstring::npos)
		{
			success = false;
		}
		else
		{
			// We have a valid start tag!
			outTag = inString.substr(tagStartIndex, tagEndIndex - tagStartIndex + 1);
			ioFirstAvailableCharIndex = tagEndIndex + 1;
		}

		return success;
	}

	bool GetTagName(const std::wstring& inTag, std::wstring& outName)
	{
		// Ignore any leading or trailing whitespace...
		std::wstring::size_type nameStartIndex = inTag.find_first_not_of(L"< \n\r\t");
		std::wstring::size_type nameEndIndex = std::wstring::npos;
		if (nameStartIndex != std::wstring::npos)
		{
			nameEndIndex = inTag.find_first_of(L" \n\r\t>", nameStartIndex);
		}

		if (nameStartIndex == std::wstring::npos || nameEndIndex == std::wstring::npos)
		{
			return false;
		}

		// We have a valid element name!
		outName = inTag.substr(nameStartIndex, nameEndIndex - nameStartIndex);
		return true;
	}

}	// private namespace



namespace AuditionSessionSDK
{


XmlElement::XmlElement()
	:
	mElementName(),
	mValue(),
	mElements()
{
}

XmlElement::XmlElement(const std::wstring& inName)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
}

XmlElement::XmlElement(const std::wstring& inName, const std::wstring& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const __int64& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const int& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const unsigned int& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const unsigned long& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const double& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::XmlElement(const std::wstring& inName, const bool& inValue)
	:
	mElementName(inName),
	mValue(),
	mElements()
{
	SetValue(inValue);
}

XmlElement::~XmlElement()
{
}

bool XmlElement::ParseInto(const std::wstring& inString, std::wstring::size_type& ioFirstAvailableCharIndex)
{
	bool success = ioFirstAvailableCharIndex + 1 < inString.size();

	std::wstring::size_type newFirstAvailableCharIndex = ioFirstAvailableCharIndex;
	std::wstring::size_type childElementFirstAvailableCharIndex = ioFirstAvailableCharIndex;

	std::wstring beginTag;
	if (success)
	{
		success = GetNextTag(inString, newFirstAvailableCharIndex, beginTag);
	}
	if (success)
	{
		childElementFirstAvailableCharIndex = newFirstAvailableCharIndex;
		success = GetTagName(beginTag, mElementName);

		if (!success || mElementName.empty() || mElementName[0] == L'/')
		{
			// not a valid start tag - this is an end tag!
			success = false;
		}

		// todo:  parse attributes...
	}

	

	// now parse the value or any child elements...
	// For this parser, an element can either have a child value or child elements, but not both...
	if (success)
	{
		std::wstring::size_type tempNextAvailableCharIndex = childElementFirstAvailableCharIndex;
		if (GetNextValue(inString, tempNextAvailableCharIndex, mValue))
		{
			// Valid child value!
			newFirstAvailableCharIndex = childElementFirstAvailableCharIndex = tempNextAvailableCharIndex;
		}
		else
		{
			// No child value...check for child elements.
			mValue.clear();

			bool childSuccess = true;
			while (childSuccess)
			{
				XmlElement childElement;
				childSuccess = childElement.ParseInto(inString, childElementFirstAvailableCharIndex);
				if (childSuccess)
				{
					mElements[childElement.GetName()] = childElement;
					newFirstAvailableCharIndex = childElementFirstAvailableCharIndex;
				}
			}
		}
	}

	// now find the end tag.
	std::wstring endTag;
	if (success)
	{
		success = GetNextTag(inString, newFirstAvailableCharIndex, endTag);
	}
	if (success)
	{
		std::wstring endTagName;
		success = GetTagName(endTag, endTagName);

		std::wstring tempElementName = mElementName;
		tempElementName.insert(0, L'/');
		if (tempElementName.compare(endTagName) != 0)
		{
			// Not the correct end tag!
			success = false;
		}
	}


	if (success)
	{
		ioFirstAvailableCharIndex = newFirstAvailableCharIndex;
	}
	else
	{
		mElementName.clear();
		mValue.clear();
		//mAttributes.clear();
		mElements.clear();
	}

	return success;
}

namespace
{
	// Very simple way to get space padding the xml file (to make more readable)
	int gPadding = 0;

	void InsertPadding(std::wstring& ioFileString)
	{
		for (int x = 0; x < gPadding; ++x)
		{
			ioFileString.append(L" ", 1);
		}
	}
}

bool XmlElement::WriteToString(std::wstring& ioFileString) const
{
	InsertPadding(ioFileString);
	ioFileString.append(L"<");
	ioFileString.append(mElementName);
	ioFileString.append(L">\r\n");

	++gPadding;

	if (!mValue.empty())
	{
		InsertPadding(ioFileString);
		ioFileString.append(mValue);
		ioFileString.append(L"\r\n");
	}
	else
	{
		ElementMap::const_iterator iter = mElements.begin();
		ElementMap::const_iterator iterEnd = mElements.end();
		for ( ; iter != iterEnd; ++iter)
		{
			iter->second.WriteToString(ioFileString);
		}
	}

	--gPadding;

	InsertPadding(ioFileString);
	ioFileString.append(L"<");
	ioFileString.append(L"/");
	ioFileString.append(mElementName);
	ioFileString.append(L">\r\n");

	return true;
}


void XmlElement::SetName(const std::wstring& inName)
{
	mElementName = inName;
}

std::wstring XmlElement::GetName() const
{
	return mElementName;
}

namespace {

	void ReplaceAllStringInString(std::wstring& ioStringToChange, 
		const std::wstring& inFindString,
		const std::wstring& inReplaceString)
	{
		std::wstring::size_type subStrPos = ioStringToChange.find(inFindString, 0);
		while(subStrPos != std::wstring::npos)
		{
			ioStringToChange.replace(subStrPos, inFindString.length(), inReplaceString);
			subStrPos = ioStringToChange.find(inFindString, subStrPos + inReplaceString.length()); 
		}
	}

	void EncodeReservedCharacters(std::wstring& inString)
	{
		ReplaceAllStringInString(inString, L"&", L"&amp;");		// this one must be first!
		ReplaceAllStringInString(inString, L"<", L"&lt;");
		ReplaceAllStringInString(inString, L">", L"&gt;");
		ReplaceAllStringInString(inString, L"\'", L"&apos;");
		ReplaceAllStringInString(inString, L"\"", L"&quot;");
	}

	void DecodeReservedCharacters(std::wstring& inString)
	{
		ReplaceAllStringInString(inString, L"&lt;", L"<");
		ReplaceAllStringInString(inString, L"&gt;", L">");
		ReplaceAllStringInString(inString, L"&amp;", L"&");
		ReplaceAllStringInString(inString, L"&apos;", L"\'");
		ReplaceAllStringInString(inString, L"&quot;", L"\"");
	}

}

void XmlElement::SetValue(const std::wstring& inValue)
{
	mValue = inValue;
	EncodeReservedCharacters(mValue);
}

bool XmlElement::GetValue(std::wstring& outString) const
{
	outString = mValue;
	DecodeReservedCharacters(outString);
	return true;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, std::wstring& outString) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outString))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const __int64& inValue)
{
	// The help says that the maximum string length returned from _i64tow
	// is 65 characters, plus null character, plus possible minus sign (if negative)
	wchar_t string[70];
	mValue = _i64tow(inValue, string, 10);
}

bool XmlElement::GetValue(__int64& outValue) const
{
	if (mValue.empty())
	{
		outValue = 0;
		return false;
	}

	outValue = _wtoi64(mValue.c_str());
	return true;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, __int64& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const int& inValue)
{
	SetValue(static_cast<__int64>(inValue));
}

bool XmlElement::GetValue(int& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = static_cast<int>(tempValue);
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, int& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const unsigned short& inValue)
{
	SetValue(static_cast<__int64>(inValue));
}

bool XmlElement::GetValue(unsigned short& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = static_cast<unsigned short>(tempValue);
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, unsigned short& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const unsigned char& inValue)
{
	SetValue(static_cast<__int64>(inValue));
}

bool XmlElement::GetValue(unsigned char& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = static_cast<unsigned char>(tempValue);
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, unsigned char& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const unsigned int& inValue)
{
	SetValue(static_cast<__int64>(inValue));
}

bool XmlElement::GetValue(unsigned int& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = static_cast<unsigned int>(tempValue);
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, unsigned int& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const unsigned long& inValue)
{
	SetValue(static_cast<__int64>(inValue));
}

bool XmlElement::GetValue(unsigned long& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = static_cast<unsigned long>(tempValue);
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, unsigned long& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const double& inValue)
{
	// The help says that the maximum string length returned from _i64tow
	// is 65 characters, plus null character, plus possible minus sign (if negative)
	char string[_CVTBUFSIZE];
	_gcvt(inValue, 20, string);

	int wstringLen = MultiByteToWideChar(CP_ACP, 0, string, strlen(string) + 1, NULL, 0);

	if (wstringLen > 0)
	{
		wchar_t* tempWideString = new wchar_t[wstringLen + 1];

		MultiByteToWideChar(CP_ACP, 0, string, strlen(string) + 1, tempWideString, wstringLen);

		mValue = tempWideString;
		delete[] tempWideString;
	}
	else
	{
		assert(false);
		mValue.clear();
	}
}

bool XmlElement::GetValue(double& outValue) const
{
	if (mValue.empty())
	{
		outValue = 0.0;
		return false;
	}

	outValue = _wtof(mValue.c_str());
	return true;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, double& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

void XmlElement::SetValue(const bool& inValue)
{
	SetValue(static_cast<__int64>(inValue ? 1 : 0));
}

bool XmlElement::GetValue(bool& outValue) const
{
	__int64 tempValue;
	bool success = GetValue(tempValue);
	if (success)
	{
		outValue = tempValue != 0;
	}
	return success;
}

bool XmlElement::GetElementValue(const std::wstring& inElementName, bool& outValue) const
{
	ElementIter iter;
	if (GetElement(inElementName, iter))
	{
		if (iter->second.GetValue(outValue))
		{
			return true;
		}
	}
	return false;
}

bool XmlElement::GetElement(const std::wstring& inElementName, ElementIter& outElementIter) const
{
	ElementMap::const_iterator iter = mElements.find(inElementName);
	if (iter != mElements.end())
	{
		outElementIter = iter;
		return true;
	}

	return false;
}

XmlElement& XmlElement::SetElement(const XmlElement& inElement)
{
	return mElements[inElement.GetName()] = inElement;
}



SimpleParser::SimpleParser()
	:
	element()
{
}

SimpleParser::~SimpleParser()
{
}

bool SimpleParser::Understands(const std::wstring& inFileName)
{
	bool understands = false;

	if (PathFileExists(inFileName.c_str()))
	{
		HANDLE hFile = CreateFile(inFileName.c_str(), GENERIC_READ, 0,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);

		if (hFile != INVALID_HANDLE_VALUE)
		{
			long dwHereHigh = 0;
			DWORD dwHere = SetFilePointer(hFile, 0, &dwHereHigh, FILE_CURRENT);
			DWORD dwSize = SetFilePointer(hFile, 0, NULL, FILE_END);
			DWORD dwTextSize = dwSize - 3;		// remove 3 bytes for the byte-order mark

			SetFilePointer(hFile, dwHere, &dwHereHigh, FILE_BEGIN);

			DWORD numChars = dwTextSize / sizeof(char);

			const std::string kBeginTag(kXmlBeginTag + "<AuditionSession>");
			DWORD numCharsInBeginTag = kBeginTag.size();

			if (numChars > numCharsInBeginTag)
			{
				DWORD dwDidRead = 0;

				// First, read the byte-order mark
				BYTE byteOrderMark;
				BYTE byteOrderMark2;
				BYTE byteOrderMark3;
				ReadFile(hFile, &byteOrderMark, 1, &dwDidRead, NULL);
				ReadFile(hFile, &byteOrderMark2, 1, &dwDidRead, NULL);
				ReadFile(hFile, &byteOrderMark3, 1, &dwDidRead, NULL);
				if (byteOrderMark == 0xEF && byteOrderMark2 == 0xBB && byteOrderMark3 == 0xBF)
				{
					char* szFile = new char[numCharsInBeginTag + 1];
					ReadFile(hFile, szFile, (numCharsInBeginTag) * sizeof(std::string::value_type), &dwDidRead, NULL);
					szFile[numCharsInBeginTag] = L'\0';		// ending null char

					if (kBeginTag.compare(szFile) == 0)
					{
						// found the appropriate beginning tag
						understands = true;
					}

					delete szFile;
				}

			}
			CloseHandle(hFile);
		}
	}

	return understands;
}

bool SimpleParser::LoadFromFile(const std::wstring& inFileName)
{
	bool returnVal = false;
	std::string fileString;

	if (PathFileExists(inFileName.c_str()))
	{
		HANDLE hFile = CreateFile(inFileName.c_str(), GENERIC_READ, 0,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);

		if (hFile != INVALID_HANDLE_VALUE)
		{
			long dwHereHigh = 0;
			DWORD dwHere = SetFilePointer(hFile, 0, &dwHereHigh, FILE_CURRENT);
			DWORD dwSize = SetFilePointer(hFile, 0, NULL, FILE_END);
			DWORD dwTextSize = dwSize - 3;		// remove 3 bytes for the byte-order mark

			SetFilePointer(hFile, dwHere, &dwHereHigh, FILE_BEGIN);

			DWORD numChars = dwTextSize / sizeof(char);

			if (numChars > 0)
			{
				DWORD dwDidRead = 0;

				// First, read the byte-order mark
				BYTE byteOrderMark;
				BYTE byteOrderMark2;
				BYTE byteOrderMark3;
				ReadFile(hFile, &byteOrderMark, 1, &dwDidRead, NULL);
				ReadFile(hFile, &byteOrderMark2, 1, &dwDidRead, NULL);
				ReadFile(hFile, &byteOrderMark3, 1, &dwDidRead, NULL);
				if (byteOrderMark == 0xEF && byteOrderMark2 == 0xBB && byteOrderMark3 == 0xBF)
				{
					char* szFile = new char[numChars + 1];
					ReadFile(hFile, szFile, dwTextSize, &dwDidRead, NULL);

					fileString = szFile;
					delete[] szFile;
				}
			}
			CloseHandle(hFile);
		}
	}

	if (!fileString.empty())
	{
		std::wstring wideString;
		wchar_t* szWideString = new wchar_t[fileString.size() + 2];

		int numWideChars = MultiByteToWideChar(CP_UTF8, NULL, fileString.c_str(), fileString.size(), szWideString, fileString.size() + 1);
		if (numWideChars)
		{
			// Add null terminating character just in case.  (MultiByteToWideChar does not guarantee the resulting string will have the null terminator)
			szWideString[numWideChars] = '\0';
			wideString = szWideString;
		}
		delete [] szWideString;

		if (!wideString.empty())
		{
			std::string::size_type firstAvailableCharacter = 0;

			// First read the standard xml beginning tag
			std::wstring xmlBeginTag = wideString.substr(0, kXmlBeginTagWide.size());
			if (xmlBeginTag.compare(kXmlBeginTagWide) == 0)
			{
				firstAvailableCharacter += kXmlBeginTagWide.size();
				returnVal = element.ParseInto(wideString, firstAvailableCharacter);
			}
		}
	}

	return returnVal;
}

bool SimpleParser::WriteToFile(const std::wstring& inFileName)
{
	bool success = false;

	HANDLE hFile = CreateFile(inFileName.c_str(), GENERIC_WRITE, 0,
		NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);

	if (hFile != INVALID_HANDLE_VALUE)
	{
		// First write out the byte-order mark
		DWORD dwWrote;
		BYTE byteOrderMark = 0xEF;
		BYTE byteOrderMark2 = 0xBB;
		BYTE byteOrderMark3 = 0xBF;
		WriteFile(hFile, &byteOrderMark, sizeof(BYTE), &dwWrote, NULL);
		WriteFile(hFile, &byteOrderMark2, sizeof(BYTE), &dwWrote, NULL);
		WriteFile(hFile, &byteOrderMark3, sizeof(BYTE), &dwWrote, NULL);

		// Next write out the standard xml beginning tag
		WriteFile(hFile, kXmlBeginTag.c_str(), sizeof(std::string::value_type) * kXmlBeginTag.size(), &dwWrote, NULL);

		// Next write out the xml data
		gPadding = 0;

		std::wstring fileString;
		success = element.WriteToString(fileString);

		if (success)
		{
			// First, get the multibyte strings for writing to the file
			int szFileString_BufferSize = ::WideCharToMultiByte(CP_UTF8, 0, fileString.c_str(), fileString.length(), 0, 0, NULL, NULL);
			char* szFileString = new char[szFileString_BufferSize + 1];
			int szFileString_NumBytes = ::WideCharToMultiByte(CP_UTF8, 0, fileString.c_str(), fileString.length(), szFileString, szFileString_BufferSize, NULL, NULL);

			WriteFile(hFile, szFileString, szFileString_NumBytes, &dwWrote, NULL);
		}

		CloseHandle(hFile);
	}

	return success;
}


}	// namespace AuditionSessionSDK

	