/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2003-2004 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#define STRICT

#include <windows.h>
#include <assert.h>

#include <strsafe.h>

#include "XML.h"
#include "..\SessionSdkInterpretter.h"

using namespace AuditionSessionSDK;


/************************************

  Audition may call PlugInSessionGetName() at any time (to get display name for this format, 
   and get the filename extension).

  PlugInSessionNew is called whenever a file is to be read or written or just 
   read for query purposes.  After an object is returned, all further communications happens
   through that object.  GetFlags() and Understands() are called when testing to see if a 
   particular file can be opened by this interpreter.

  WHEN WRITING A FILE:
	Open(szFilename, PLUGINSES_OF_WRITE)
	SetChunkInfo(...)
	...
	SetChunkInfo(...)
	Close()

   WHEN READING A FILE:
    Open(szFilename, PLUGINSES_OF_READ)
	GetChunkInfo(...)
	...
	GetChunkInfo(...)
	Close()

   In general, the plug-in should be written to store up all the necessary information needed
   to write in the destination format during the SetChunkInfo() calls, then at Close(), write
   the entire file in the proper format, now that we have all the information needed.

   When reading a file, in the Open() call, read in and store all relavent information, then
   fill out the appropriate structures when asked for them in the GetChunkInfo() calls.  Alternately,
   you can fill out copies of all possible chunks, then copy from them when asked for the info
   in the GetChunkInfo() calls.


*************************************/


namespace
{
const WCHAR SES_EXTENSION[] = L"xml";
const WCHAR SES_NAME[] = L"XML";

#define SES_EXTENSION_LEN	3
#define SES_NAME_LEN		3
}

CPlugInSession * WINAPI PlugInSessionNew(void)
{
	return (CPlugInSession *)new XMLSession();
}



/*
 ** Spawn
 *
 *  FILENAME: 		text.cpp
 *
 *  FUNCTION:		HRESULT WINAPI PlugInSessionGetName(LPSESSION_NAME_INFO lpSessionNameInfo);
 *
 *  PARAMETERS:		Pointer to SESSION_NAME_INFO struct
 *
 *  DESCRIPTION:	Populates SESSION_NAME_INFO structure
 *
 *  RETURNS:		Populates strings members. 
 *					Returns length of strings copied into their associated members on SUCCESS, 0 otherwise.
 *
 */

HRESULT WINAPI PlugInSessionGetName(LPSESSION_NAME_INFO lpSessionNameInfo)
{
	HRESULT hr = E_FAIL;

	if (lpSessionNameInfo->dwSize >= sizeof(SESSION_NAME_INFO))
	{
		if (lpSessionNameInfo->lpszSessionName)
			lpSessionNameInfo->nSessionNameLen = (SUCCEEDED(hr = StringCchCopyN(lpSessionNameInfo->lpszSessionName, lpSessionNameInfo->nSessionNameLen, SES_NAME, SES_NAME_LEN)))		 ? SES_NAME_LEN		 : 0;

		if (lpSessionNameInfo->lpszSessionExt)
			lpSessionNameInfo->nSessionExtLen  = (SUCCEEDED(hr = StringCchCopyN(lpSessionNameInfo->lpszSessionExt, lpSessionNameInfo->nSessionExtLen, SES_EXTENSION, SES_EXTENSION_LEN ))) ? SES_EXTENSION_LEN : 0;
	} 

	return hr;
}


XMLSession::XMLSession()
	:
	mInterpretterToSdk(NULL),
	mSessionFilename(),
	m_dwFlags(PLUGINSES_CANOPEN|PLUGINSES_CANSAVE)
{
}

XMLSession::~XMLSession()
{
	CloseSession();
}

HRESULT XMLSession::GetFlags(DWORD * pdwFlags)
{
	*pdwFlags = m_dwFlags;
	return S_OK;
}

HRESULT XMLSession::Understands(LPCTSTR szFilename)
{
	SimpleParser parser;
	return parser.Understands(szFilename) ? S_OK : E_FAIL;
}

HRESULT	XMLSession::ReadSession(LPCTSTR szFilename, FULL_SDK_SESSION_INFO** outSessionInfo)
{
	CloseSession();

	HRESULT hr = E_FAIL;

	mSessionFilename = szFilename;

	COMPLETE_SESSION_INFO_NEW sessionInfo;
	hr = ReadEntireSession(sessionInfo) ? S_OK : E_FAIL;

	if (S_OK == hr)
	{

		mInterpretterToSdk = new AuditionSessionSDK::SessionIterpretter_ToSDK;
		*outSessionInfo = mInterpretterToSdk->FormatSessionDataForSdk(sessionInfo);
	}
	else
	{
		mSessionFilename.clear();
	}

	return hr;
}

HRESULT XMLSession::CloseSession(void)
{
	if (mInterpretterToSdk != NULL)
	{
		delete mInterpretterToSdk;
		mInterpretterToSdk = NULL;
	}

	return S_OK;
}

HRESULT	XMLSession::WriteSession(LPCTSTR szFilename, FULL_SDK_SESSION_INFO* inSessionInfo)
{
	mSessionFilename = szFilename;

	AuditionSessionSDK::SessionIterpretter_FromSDK interpretterFromSdk;
	COMPLETE_SESSION_INFO_NEW& sessionInfo = interpretterFromSdk.GetSessionDataFromSdk(inSessionInfo);

	return WriteEntireSession(sessionInfo) ? S_OK : E_FAIL;
}

